@@ -15,6 +15,7 @@ import (
 	"strings"
 	"text/template"
 
+	"ariga.io/atlas/cmd/atlas/internal/cmdext"
 	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
@@ -25,18 +26,6 @@ import (
 	"github.com/spf13/cobra"
 )
 
-func init() {
-	schemaCmd := schemaCmd()
-	schemaCmd.AddCommand(
-		schemaApplyCmd(),
-		schemaCleanCmd(),
-		schemaDiffCmd(),
-		schemaFmtCmd(),
-		schemaInspectCmd(),
-	)
-	Root.AddCommand(schemaCmd)
-}
-
 // schemaCmd represents the subcommand 'atlas schema'.
 func schemaCmd() *cobra.Command {
 	cmd := &cobra.Command{
@@ -58,6 +47,7 @@ type schemaApplyFlags struct {
 	dryRun      bool     // Only show SQL on screen instead of applying it.
 	autoApprove bool     // Don't prompt for approval before applying SQL.
 	logFormat   string   // Log format.
+	txMode      string   // (none, file)
 }
 
 // schemaApplyCmd represents the 'atlas schema apply' subcommand.
@@ -118,6 +108,7 @@ migration.`,
 	addFlagAutoApprove(cmd.Flags(), &flags.autoApprove)
 	addFlagLog(cmd.Flags(), &flags.logFormat)
 	addFlagFormat(cmd.Flags(), &flags.logFormat)
+	cmd.Flags().StringVarP(&flags.txMode, flagTxMode, "", txModeFile, "set transaction mode [none, file]")
 	// Hidden support for the deprecated -f flag.
 	cmd.Flags().StringSliceVarP(&flags.paths, flagFile, "f", nil, "[paths...] file or directory containing HCL or SQL files")
 	cobra.CheckErr(cmd.Flags().MarkHidden(flagFile))
@@ -134,6 +125,8 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 		return errors.New(`one of flag(s) "file" or "to" is required`)
 	case flags.logFormat != "" && !flags.dryRun && !flags.autoApprove:
 		return errors.New(`--log and --format can only be used with --dry-run or --auto-approve`)
+	case flags.txMode != txModeNone && flags.txMode != txModeFile:
+		return fmt.Errorf("unknown tx-mode %q", flags.txMode)
 	}
 	// If the old -f flag is given convert them to the URL format. If both are given,
 	// cobra would throw an error since they are marked as mutually exclusive.
@@ -173,6 +166,22 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 	if !ok {
 		return errors.New("--url must be a database connection")
 	}
+	apply := client.ApplyChanges
+	if !flags.dryRun && flags.txMode == txModeFile {
+		apply = func(ctx context.Context, cs []schema.Change, _ ...migrate.PlanOption) error {
+			tx, err := client.Tx(ctx, nil)
+			if err != nil {
+				return err
+			}
+			if err := tx.ApplyChanges(ctx, cs); err != nil {
+				// Rollback on error but the underlying error
+				// is still returned to make type-assertion pass.
+				_ = tx.Rollback()
+				return err
+			}
+			return tx.Commit()
+		}
+	}
 	to, err := stateReader(ctx, &stateReaderConfig{
 		urls:    flags.toURLs,
 		dev:     dev,
@@ -205,27 +214,21 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 		if plan, err = client.PlanChanges(ctx, "", changes); err != nil {
 			return err
 		}
-		if err = client.ApplyChanges(ctx, changes); err == nil {
+		if err = apply(ctx, changes); err == nil {
 			applied = len(plan.Changes)
 		} else if i, ok := err.(interface{ Applied() int }); ok && i.Applied() < len(plan.Changes) {
 			applied, cause = i.Applied(), &cmdlog.StmtError{Stmt: plan.Changes[i.Applied()].Cmd, Text: err.Error()}
 		} else {
 			cause = &cmdlog.StmtError{Text: err.Error()}
 		}
-		apply := cmdlog.NewSchemaApply(cmdlog.NewEnv(client, nil), plan.Changes[:applied], plan.Changes[applied:], cause)
-		if err1 := format.Execute(out, apply); err1 != nil {
-			if err != nil {
-				err1 = fmt.Errorf("%w: %v", err, err1)
-			}
-			err = err1
-		}
-		return err
+		err1 := format.Execute(out, cmdlog.NewSchemaApply(cmdlog.NewEnv(client, nil), plan.Changes[:applied], plan.Changes[applied:], cause))
+		return errors.Join(err, err1)
 	default:
 		if err := summary(cmd, client, changes, format); err != nil {
 			return err
 		}
 		if !flags.dryRun && (flags.autoApprove || promptUser()) {
-			return client.ApplyChanges(ctx, changes)
+			return apply(ctx, changes)
 		}
 		return nil
 	}
@@ -317,6 +320,11 @@ type schemaDiffFlags struct {
 
 // schemaDiffCmd represents the 'atlas schema diff' subcommand.
 func schemaDiffCmd() *cobra.Command {
+	cmd, _ := schemaDiffCmdWithFlags()
+	return cmd
+}
+
+func schemaDiffCmdWithFlags() (*cobra.Command, *schemaDiffFlags) {
 	var (
 		flags schemaDiffFlags
 		cmd   = &cobra.Command{
@@ -350,7 +358,7 @@ The database states can be read from a connected database, an HCL project or a m
 	addFlagFormat(cmd.Flags(), &flags.format)
 	cobra.CheckErr(cmd.MarkFlagRequired(flagFrom))
 	cobra.CheckErr(cmd.MarkFlagRequired(flagTo))
-	return cmd
+	return cmd, &flags
 }
 
 func schemaDiffRun(cmd *cobra.Command, _ []string, flags schemaDiffFlags, env *Env) error {
@@ -420,6 +428,11 @@ type schemaInspectFlags struct {
 
 // schemaInspectCmd represents the 'atlas schema inspect' subcommand.
 func schemaInspectCmd() *cobra.Command {
+	cmd, _ := schemaInspectCmdWithFlags()
+	return cmd
+}
+
+func schemaInspectCmdWithFlags() (*cobra.Command, *schemaInspectFlags) {
 	var (
 		flags schemaInspectFlags
 		cmd   = &cobra.Command{
@@ -459,7 +472,7 @@ flag.
 	addFlagFormat(cmd.Flags(), &flags.logFormat)
 	cobra.CheckErr(cmd.MarkFlagRequired(flagURL))
 	cmd.MarkFlagsMutuallyExclusive(flagLog, flagFormat)
-	return cmd
+	return cmd, &flags
 }
 
 func schemaInspectRun(cmd *cobra.Command, _ []string, flags schemaInspectFlags) error {
@@ -467,10 +480,12 @@ func schemaInspectRun(cmd *cobra.Command, _ []string, flags schemaInspectFlags)
 		ctx = cmd.Context()
 		dev *sqlclient.Client
 	)
-	if flags.devURL != "" {
-		var err error
-		dev, err = sqlclient.Open(ctx, flags.devURL)
-		if err != nil {
+	useDev, err := readerUseDev(flags.url)
+	if err != nil {
+		return err
+	}
+	if flags.devURL != "" && useDev {
+		if dev, err = sqlclient.Open(ctx, flags.devURL); err != nil {
 			return err
 		}
 		defer dev.Close()
@@ -620,7 +635,7 @@ func setSchemaEnvFlags(cmd *cobra.Command, env *Env) error {
 	return nil
 }
 
-func computeDiff(ctx context.Context, differ *sqlclient.Client, from, to *stateReadCloser, opts ...schema.DiffOption) ([]schema.Change, error) {
+func computeDiff(ctx context.Context, differ *sqlclient.Client, from, to *cmdext.StateReadCloser, opts ...schema.DiffOption) ([]schema.Change, error) {
 	current, err := from.ReadState(ctx)
 	if err != nil {
 		return nil, err
@@ -632,16 +647,16 @@ func computeDiff(ctx context.Context, differ *sqlclient.Client, from, to *stateR
 	var diff []schema.Change
 	switch {
 	// In case an HCL file is compared against a specific database schema (not a realm).
-	case to.hcl && len(desired.Schemas) == 1 && from.schema != "" && desired.Schemas[0].Name != from.schema:
-		return nil, fmt.Errorf("mismatched HCL and database schemas: %q <> %q", from.schema, desired.Schemas[0].Name)
+	case to.HCL && len(desired.Schemas) == 1 && from.Schema != "" && desired.Schemas[0].Name != from.Schema:
+		return nil, fmt.Errorf("mismatched HCL and database schemas: %q <> %q", from.Schema, desired.Schemas[0].Name)
 	// Compare realm if the desired state is an HCL file or both connections are not bound to a schema.
-	case from.hcl, to.hcl, from.schema == "" && to.schema == "":
+	case from.HCL, to.HCL, from.Schema == "" && to.Schema == "":
 		diff, err = differ.RealmDiff(current, desired, opts...)
 		if err != nil {
 			return nil, err
 		}
-	case from.schema == "", to.schema == "":
-		return nil, fmt.Errorf("cannot diff a schema with a database connection: %q <> %q", from.schema, to.schema)
+	case from.Schema == "", to.Schema == "":
+		return nil, fmt.Errorf("cannot diff a schema with a database connection: %q <> %q", from.Schema, to.Schema)
 	default:
 		// SchemaDiff checks for name equality which is irrelevant in the case
 		// the user wants to compare their contents, reset them to allow the comparison.
