@@ -23,13 +23,18 @@ import (
 )
 
 var (
+	ColorCyan         = color.CyanString
+	ColorGreen        = color.HiGreenString
+	ColorRed          = color.HiRedString
+	ColorRedBgWhiteFg = color.New(color.FgHiWhite, color.BgHiRed).SprintFunc()
+	ColorYellow       = color.YellowString
 	// ColorTemplateFuncs are globally available functions to color strings in a report template.
 	ColorTemplateFuncs = template.FuncMap{
-		"cyan":         color.CyanString,
-		"green":        color.HiGreenString,
-		"red":          color.HiRedString,
-		"redBgWhiteFg": color.New(color.FgHiWhite, color.BgHiRed).SprintFunc(),
-		"yellow":       color.YellowString,
+		"cyan":         ColorCyan,
+		"green":        ColorGreen,
+		"red":          ColorRed,
+		"redBgWhiteFg": ColorRedBgWhiteFg,
+		"yellow":       ColorYellow,
 	}
 )
 
@@ -646,6 +651,11 @@ func (s *SchemaInspect) MarshalJSON() ([]byte, error) {
 	return json.Marshal(realm)
 }
 
+// MarshalSQL returns the default SQL representation of the schema.
+func (s *SchemaInspect) MarshalSQL(indent ...string) (string, error) {
+	return sqlInspect(s, indent...)
+}
+
 func sqlInspect(report *SchemaInspect, indent ...string) (string, error) {
 	if report.Error != nil {
 		return report.Error.Error(), nil
@@ -659,6 +669,12 @@ func sqlInspect(report *SchemaInspect, indent ...string) (string, error) {
 		for _, t := range s.Tables {
 			changes = append(changes, &schema.AddTable{T: t})
 		}
+		for _, v := range s.Views {
+			changes = append(changes, &schema.AddView{V: v})
+		}
+		for _, o := range s.Objects {
+			changes = append(changes, &schema.AddObject{O: o})
+		}
 	}
 	return fmtPlan(report.Client, changes, indent)
 }
@@ -695,6 +711,7 @@ func fmtPlan(client *sqlclient.Client, changes schema.Changes, indent []string)
 		return "", fmt.Errorf("unexpected number of arguments: %d", len(indent))
 	}
 	plan, err := client.PlanChanges(context.Background(), "plan", changes, func(o *migrate.PlanOptions) {
+		o.Mode = migrate.PlanModeDump
 		// Disable tables qualifier in schema-mode.
 		if client.URL.Schema != "" {
 			o.SchemaQualifier = new(string)
