@@ -8,6 +8,11 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"io/fs"
+	"net/url"
+	"os"
+	"path"
+	"path/filepath"
 	"time"
 
 	"ariga.io/atlas/cmd/atlas/internal/migrate/ent"
@@ -15,6 +20,8 @@ import (
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
 	"ariga.io/atlas/sql/sqlclient"
+	"ariga.io/atlas/sql/sqltool"
+
 	"entgo.io/ent/dialect"
 	"entgo.io/ent/dialect/sql"
 	entschema "entgo.io/ent/dialect/sql/schema"
@@ -185,7 +192,29 @@ func (r *EntRevisions) Migrate(ctx context.Context) (err error) {
 			}()
 		}
 	}
-	return c.Schema.Create(ctx, entschema.WithDropColumn(true))
+	return c.Schema.Create(ctx, entschema.WithDropColumn(true), entschema.WithDiffHook(func(next entschema.Differ) entschema.Differ {
+		return entschema.DiffFunc(func(current, desired *schema.Schema) ([]schema.Change, error) {
+			changes, err := next.Diff(current, desired)
+			if err != nil {
+				return nil, err
+			}
+			// Skip all changes beside revisions
+			// table creation or modification.
+			for i := range changes {
+				switch cs := changes[i].(type) {
+				case *schema.AddTable:
+					if cs.T.Name == revision.Table {
+						return schema.Changes{cs}, nil
+					}
+				case *schema.ModifyTable:
+					if cs.T.Name == revision.Table {
+						return schema.Changes{cs}, nil
+					}
+				}
+			}
+			return nil, nil
+		})
+	}))
 }
 
 // revisionID holds the column "id" ("version") of the revision that holds the identifier of the
@@ -219,3 +248,88 @@ func (r *EntRevisions) ID(ctx context.Context, operatorV string) (string, error)
 }
 
 var _ migrate.RevisionReadWriter = (*EntRevisions)(nil)
+
+// List of supported formats.
+const (
+	FormatAtlas         = "atlas"
+	FormatGolangMigrate = "golang-migrate"
+	FormatGoose         = "goose"
+	FormatFlyway        = "flyway"
+	FormatLiquibase     = "liquibase"
+	FormatDBMate        = "dbmate"
+)
+
+// Formats is the list of supported formats.
+var Formats = []string{FormatAtlas, FormatGolangMigrate, FormatGoose, FormatFlyway, FormatLiquibase, FormatDBMate}
+
+// Formatter returns the dir formatter for its URL.
+func Formatter(u *url.URL) (migrate.Formatter, error) {
+	switch f := u.Query().Get("format"); f {
+	case FormatAtlas:
+		return migrate.DefaultFormatter, nil
+	case FormatGolangMigrate:
+		return sqltool.GolangMigrateFormatter, nil
+	case FormatGoose:
+		return sqltool.GooseFormatter, nil
+	case FormatFlyway:
+		return sqltool.FlywayFormatter, nil
+	case FormatLiquibase:
+		return sqltool.LiquibaseFormatter, nil
+	case FormatDBMate:
+		return sqltool.DBMateFormatter, nil
+	default:
+		return nil, fmt.Errorf("unknown format %q", f)
+	}
+}
+
+// Dir parses u and calls dirURL.
+func Dir(u string, create bool) (migrate.Dir, error) {
+	parsed, err := url.Parse(u)
+	if err != nil {
+		return nil, err
+	}
+	return DirURL(parsed, create)
+}
+
+// DirURL returns a migrate.Dir to use as migration directory. For now only local directories are supported.
+func DirURL(u *url.URL, create bool) (migrate.Dir, error) {
+	p := filepath.Join(u.Host, u.Path)
+	switch u.Scheme {
+	case "mem":
+		return migrate.OpenMemDir(path.Join(u.Host, u.Path)), nil
+	case "file":
+		if p == "" {
+			p = "migrations"
+		}
+	default:
+		return nil, fmt.Errorf("unsupported driver %q", u.Scheme)
+	}
+	fn := func() (migrate.Dir, error) { return migrate.NewLocalDir(p) }
+	switch f := u.Query().Get("format"); f {
+	case "", FormatAtlas:
+		// this is the default
+	case FormatGolangMigrate:
+		fn = func() (migrate.Dir, error) { return sqltool.NewGolangMigrateDir(p) }
+	case FormatGoose:
+		fn = func() (migrate.Dir, error) { return sqltool.NewGooseDir(p) }
+	case FormatFlyway:
+		fn = func() (migrate.Dir, error) { return sqltool.NewFlywayDir(p) }
+	case FormatLiquibase:
+		fn = func() (migrate.Dir, error) { return sqltool.NewLiquibaseDir(p) }
+	case FormatDBMate:
+		fn = func() (migrate.Dir, error) { return sqltool.NewDBMateDir(p) }
+	default:
+		return nil, fmt.Errorf("unknown dir format %q", f)
+	}
+	d, err := fn()
+	if create && errors.Is(err, fs.ErrNotExist) {
+		if err := os.MkdirAll(p, 0755); err != nil {
+			return nil, err
+		}
+		d, err = fn()
+		if err != nil {
+			return nil, err
+		}
+	}
+	return d, err
+}
