@@ -37,7 +37,7 @@ module.exports = {
       title: 'Atlas',
       logo: {
         alt: 'Atlas',
-        src: 'https://atlasgo.io/uploads/landing/logo.svg',
+        src: 'https://atlasgo.io/uploads/websiteicon.svg',
       },
       items: [
         {
@@ -140,9 +140,11 @@ module.exports = {
       `,
     },
     announcementBar: {
-      id: 'announcementBar-3', // Increment on change
-      content: `Announcing v0.11.0! Manage database schemas with Kubernetes and Atlas, Read it <a href="https://atlasgo.io/blog/2023/05/08/atlas-v011-kubernetes-operator"> here!</a> 🚀`,
+      id: 'announcementBar-5', // Increment on change
+      content: `<strong>Upcoming Webinar:</strong> Kubernetes-native schema migrations <a class="cta" target="_blank" href="https://page.ariga.io/kubernetes-native-schema-migrations-webinar?utm_source=atlasgo&utm_medium=announcementbar">Register now</a>`,
       isCloseable: true,
+      backgroundColor: '#008CFA',
+      textColor: '#fff',
     },
   },
   plugins: [
@@ -187,11 +189,11 @@ module.exports = {
             from: '/cli/projects',
           },
           {
-            to: '/atlas-schema/sql-types',
+            to: '/atlas-schema/hcl-types',
             from: '/ddl/sql-types',
           },
           {
-            to: '/atlas-schema/sql-resources',
+            to: '/atlas-schema/hcl',
             from: '/ddl/sql',
           },
           {
@@ -218,6 +220,14 @@ module.exports = {
             to: '/concepts/url',
             from: '/url',
           },
+          {
+            from: '/atlas-schema/sql-resources',
+            to: '/atlas-schema/hcl',
+          },
+          {
+            from: '/atlas-schema/sql-types',
+            to: '/atlas-schema/hcl-types',
+          },
         ],
       },
     ],
@@ -227,6 +237,16 @@ module.exports = {
         id: 'GTM-T9GX8BR', // GTM Container ID
       }
     ],
+    [
+      '@docusaurus/plugin-ideal-image',
+      {
+        quality: 70,
+        max: 1030,
+        min: 640,
+        steps: 2,
+        disableInDev: false,
+      },
+    ],
   ],
   presets: [
     [
