@@ -9,7 +9,6 @@ import (
 	"context"
 	"database/sql"
 	"flag"
-	"fmt"
 	"io"
 	"os"
 	"os/exec"
@@ -77,12 +76,19 @@ func testAddDrop(t T) {
 	petsT.ForeignKeys = []*schema.ForeignKey{
 		{Symbol: "owner_id", Table: petsT, Columns: petsT.Columns[1:], RefTable: usersT, RefColumns: usersT.Columns[:1]},
 	}
+	if tt, ok := t.(interface {
+		pets(_, _ *schema.Table) *schema.Table
+	}); ok {
+		petsT = tt.pets(usersT, postsT)
+	}
 	t.dropTables(postsT.Name, usersT.Name, petsT.Name)
 	t.migrate(&schema.AddTable{T: petsT}, &schema.AddTable{T: usersT}, &schema.AddTable{T: postsT})
 	ensureNoChange(t, usersT, petsT, postsT)
 	t.migrate(&schema.DropTable{T: usersT}, &schema.DropTable{T: postsT}, &schema.DropTable{T: petsT})
-	// Ensure the realm is empty.
-	require.EqualValues(t, t.realm(), t.loadRealm())
+	// Ensure the realm has no tables.
+	for _, s := range t.loadRealm().Schemas {
+		require.Empty(t, s.Tables)
+	}
 }
 
 func testRelation(t T) {
@@ -140,7 +146,6 @@ func testHCLIntegration(t T, full string, empty string) {
 }
 
 func testCLIMigrateApplyBC(t T, dialect string) {
-	require.NoError(t, initCLI())
 	ctx := context.Background()
 
 	t.dropSchemas("bc_test", "bc_test_2", "atlas_schema_revisions")
@@ -149,7 +154,7 @@ func testCLIMigrateApplyBC(t T, dialect string) {
 
 	// Connection to schema with flag will respect flag (also mimics "old" behavior).
 	out, err := exec.Command(
-		"go", "run", "ariga.io/atlas/cmd/atlas",
+		execPath(t),
 		"migrate", "apply",
 		"--allow-dirty", // since database does contain more than one schema
 		"--dir", "file://testdata/migrations/"+dialect,
@@ -157,14 +162,16 @@ func testCLIMigrateApplyBC(t T, dialect string) {
 		"--revisions-schema", "atlas_schema_revisions",
 	).CombinedOutput()
 	require.NoError(t, err, string(out))
-	s, err := t.driver().InspectSchema(ctx, "atlas_schema_revisions", nil)
+	s, err := t.driver().InspectSchema(ctx, "atlas_schema_revisions", &schema.InspectOptions{
+		Mode: ^schema.InspectViews,
+	})
 	require.NoError(t, err)
 	_, ok := s.Table("atlas_schema_revisions")
 	require.True(t, ok)
 
 	// Connection to realm will see the existing schema and will not attempt to migrate.
 	out, err = exec.Command(
-		"go", "run", "ariga.io/atlas/cmd/atlas",
+		execPath(t),
 		"migrate", "apply",
 		"--dir", "file://testdata/migrations/"+dialect,
 		"--url", t.url(""),
@@ -174,7 +181,7 @@ func testCLIMigrateApplyBC(t T, dialect string) {
 
 	// Connection to schema without flag will error.
 	out, err = exec.Command(
-		"go", "run", "ariga.io/atlas/cmd/atlas",
+		execPath(t),
 		"migrate", "apply",
 		"--dir", "file://testdata/migrations/"+dialect,
 		"--url", t.url("bc_test"),
@@ -184,7 +191,7 @@ func testCLIMigrateApplyBC(t T, dialect string) {
 
 	// Providing the flag and we are good.
 	out, err = exec.Command(
-		"go", "run", "ariga.io/atlas/cmd/atlas",
+		execPath(t),
 		"migrate", "apply",
 		"--dir", "file://testdata/migrations/"+dialect,
 		"--url", t.url("bc_test"),
@@ -199,7 +206,7 @@ func testCLIMigrateApplyBC(t T, dialect string) {
 		&schema.AddSchema{S: schema.New("bc_test")},
 	)
 	out, err = exec.Command(
-		"go", "run", "ariga.io/atlas/cmd/atlas",
+		execPath(t),
 		"migrate", "apply",
 		"--dir", "file://testdata/migrations/"+dialect,
 		"--url", t.url("bc_test"),
@@ -210,7 +217,7 @@ func testCLIMigrateApplyBC(t T, dialect string) {
 
 	// Consecutive attempts do not need the flag anymore.
 	out, err = exec.Command(
-		"go", "run", "ariga.io/atlas/cmd/atlas",
+		execPath(t),
 		"migrate", "apply",
 		"--dir", "file://testdata/migrations/"+dialect,
 		"--url", t.url("bc_test"),
@@ -226,37 +233,39 @@ func testCLIMigrateApplyBC(t T, dialect string) {
 		&schema.AddSchema{S: schema.New("bc_test_2")},
 	)
 	out, err = exec.Command(
-		"go", "run", "ariga.io/atlas/cmd/atlas",
+		execPath(t),
 		"migrate", "apply",
 		"--allow-dirty", // since database does contain more than one schema
 		"--dir", "file://testdata/migrations/"+dialect,
 		"--url", t.url("bc_test_2"),
 	).CombinedOutput()
 	require.NoError(t, err, string(out))
-	s, err = t.driver().InspectSchema(ctx, "atlas_schema_revisions", nil)
+	s, err = t.driver().InspectSchema(ctx, "atlas_schema_revisions", &schema.InspectOptions{
+		Mode: ^schema.InspectViews,
+	})
 	require.True(t, schema.IsNotExistError(err))
-	s, err = t.driver().InspectSchema(ctx, "bc_test_2", nil)
+	s, err = t.driver().InspectSchema(ctx, "bc_test_2", &schema.InspectOptions{
+		Mode: ^schema.InspectViews,
+	})
 	require.NoError(t, err)
 	_, ok = s.Table("atlas_schema_revisions")
 	require.True(t, ok)
 }
 
 func testCLISchemaInspect(t T, h string, url string, eval schemahcl.Evaluator, args ...string) {
-	require.NoError(t, initCLI())
 	t.dropTables("users")
 	var expected schema.Schema
 	err := evalBytes([]byte(h), &expected, eval)
 	require.NoError(t, err)
 	t.applyHcl(h)
 	runArgs := []string{
-		"run", "ariga.io/atlas/cmd/atlas",
 		"schema",
 		"inspect",
 		"-u",
 		url,
 	}
 	runArgs = append(runArgs, args...)
-	cmd := exec.Command("go", runArgs...)
+	cmd := exec.Command(execPath(t), runArgs...)
 	stdout, stderr := bytes.NewBuffer(nil), bytes.NewBuffer(nil)
 	cmd.Stderr = stderr
 	cmd.Stdout = stdout
@@ -269,14 +278,12 @@ func testCLISchemaInspect(t T, h string, url string, eval schemahcl.Evaluator, a
 }
 
 func testCLISchemaInspectEnv(t T, h string, env string, eval schemahcl.Evaluator) {
-	err := initCLI()
-	require.NoError(t, err)
 	t.dropTables("users")
 	var expected schema.Schema
-	err = evalBytes([]byte(h), &expected, eval)
+	err := evalBytes([]byte(h), &expected, eval)
 	require.NoError(t, err)
 	t.applyHcl(h)
-	cmd := exec.Command("go", "run", "ariga.io/atlas/cmd/atlas",
+	cmd := exec.Command(execPath(t),
 		"schema",
 		"inspect",
 		"--env",
@@ -296,24 +303,30 @@ func testCLISchemaInspectEnv(t T, h string, env string, eval schemahcl.Evaluator
 // initOnce controls that the cli will only be built once.
 var initOnce sync.Once
 
-func initCLI() error {
-	var err error
+func execPath(t testing.TB) string {
 	initOnce.Do(func() {
-		err = exec.Command("go", "run", "-mod=mod", "ariga.io/atlas/cmd/atlas").Run()
+		args := []string{
+			"build",
+			"-mod=mod",
+			"-o", filepath.Join(os.TempDir(), "atlas"),
+		}
+		args = append(args, buildFlags...)
+		args = append(args, "ariga.io/atlas/cmd/atlas")
+		out, err := exec.Command("go", args...).CombinedOutput()
+		require.NoError(t, err, string(out))
 	})
-	return err
+	return filepath.Join(os.TempDir(), "atlas")
 }
 
 func testCLIMultiSchemaApply(t T, h string, url string, schemas []string, eval schemahcl.Evaluator) {
-	err := initCLI()
 	f := filepath.Join(t.TempDir(), "schema.hcl")
-	err = os.WriteFile(f, []byte(h), 0644)
+	err := os.WriteFile(f, []byte(h), 0644)
 	require.NoError(t, err)
 	require.NoError(t, err)
 	var expected schema.Realm
 	err = evalBytes([]byte(h), &expected, eval)
 	require.NoError(t, err)
-	cmd := exec.Command("go", "run", "ariga.io/atlas/cmd/atlas",
+	cmd := exec.Command(execPath(t),
 		"schema",
 		"apply",
 		"-f",
@@ -335,13 +348,11 @@ func testCLIMultiSchemaApply(t T, h string, url string, schemas []string, eval s
 }
 
 func testCLIMultiSchemaInspect(t T, h string, url string, schemas []string, eval schemahcl.Evaluator) {
-	err := initCLI()
-	require.NoError(t, err)
 	var expected schema.Realm
-	err = evalBytes([]byte(h), &expected, eval)
+	err := evalBytes([]byte(h), &expected, eval)
 	require.NoError(t, err)
 	t.applyRealmHcl(h)
-	cmd := exec.Command("go", "run", "ariga.io/atlas/cmd/atlas",
+	cmd := exec.Command(execPath(t),
 		"schema",
 		"inspect",
 		"-u",
@@ -361,14 +372,11 @@ func testCLIMultiSchemaInspect(t T, h string, url string, schemas []string, eval
 }
 
 func testCLISchemaApply(t T, h string, url string, args ...string) {
-	err := initCLI()
-	require.NoError(t, err)
 	t.dropTables("users")
 	f := filepath.Join(t.TempDir(), "schema.hcl")
-	err = os.WriteFile(f, []byte(h), 0644)
+	err := os.WriteFile(f, []byte(h), 0644)
 	require.NoError(t, err)
 	runArgs := []string{
-		"run", "ariga.io/atlas/cmd/atlas",
 		"schema",
 		"apply",
 		"-u",
@@ -379,7 +387,7 @@ func testCLISchemaApply(t T, h string, url string, args ...string) {
 		url,
 	}
 	runArgs = append(runArgs, args...)
-	cmd := exec.Command("go", runArgs...)
+	cmd := exec.Command(execPath(t), runArgs...)
 	stdout, stderr := bytes.NewBuffer(nil), bytes.NewBuffer(nil)
 	cmd.Stderr = stderr
 	cmd.Stdout = stdout
@@ -396,13 +404,11 @@ func testCLISchemaApply(t T, h string, url string, args ...string) {
 }
 
 func testCLISchemaApplyDry(t T, h string, url string) {
-	err := initCLI()
-	require.NoError(t, err)
 	t.dropTables("users")
 	f := filepath.Join(t.TempDir(), "schema.hcl")
-	err = os.WriteFile(f, []byte(h), 0644)
+	err := os.WriteFile(f, []byte(h), 0644)
 	require.NoError(t, err)
-	cmd := exec.Command("go", "run", "ariga.io/atlas/cmd/atlas",
+	cmd := exec.Command(execPath(t),
 		"schema",
 		"apply",
 		"-u",
@@ -429,14 +435,11 @@ func testCLISchemaApplyDry(t T, h string, url string) {
 }
 
 func testCLISchemaApplyAutoApprove(t T, h string, url string, args ...string) {
-	err := initCLI()
-	require.NoError(t, err)
 	t.dropTables("users")
 	f := filepath.Join(t.TempDir(), "schema.hcl")
-	err = os.WriteFile(f, []byte(h), 0644)
+	err := os.WriteFile(f, []byte(h), 0644)
 	require.NoError(t, err)
 	runArgs := []string{
-		"run", "ariga.io/atlas/cmd/atlas",
 		"schema",
 		"apply",
 		"-u",
@@ -446,7 +449,7 @@ func testCLISchemaApplyAutoApprove(t T, h string, url string, args ...string) {
 		"--auto-approve",
 	}
 	runArgs = append(runArgs, args...)
-	cmd := exec.Command("go", runArgs...)
+	cmd := exec.Command(execPath(t), runArgs...)
 	stdout, stderr := bytes.NewBuffer(nil), bytes.NewBuffer(nil)
 	cmd.Stderr = stderr
 	cmd.Stdout = stdout
@@ -496,16 +499,15 @@ func testCLISchemaApplyFromMigrationDir(t T) {
 	require.NoError(t, os.WriteFile(filepath.Join(p, "1.sql"), []byte(one), 0644))
 	require.NoError(t, os.WriteFile(filepath.Join(p, "2.sql"), []byte(two), 0644))
 
-	require.NoError(t, initCLI())
 	require.NoError(t, exec.Command(
-		"go", "run", "ariga.io/atlas/cmd/atlas",
+		execPath(t),
 		"migrate", "hash",
 		"--dir", "file://"+p,
 	).Run())
 
 	// All versions - must contain all migration files.
 	out, err := exec.Command(
-		"go", "run", "ariga.io/atlas/cmd/atlas",
+		execPath(t),
 		"schema", "apply",
 		"-u", t.url(dbname),
 		"--to", "file://"+p,
@@ -518,7 +520,7 @@ func testCLISchemaApplyFromMigrationDir(t T) {
 
 	// One version - must contain only file one.
 	out, err = exec.Command(
-		"go", "run", "ariga.io/atlas/cmd/atlas",
+		execPath(t),
 		"schema", "apply",
 		"-u", t.url(dbname),
 		"--to", "file://"+p+"?version=1",
@@ -531,11 +533,8 @@ func testCLISchemaApplyFromMigrationDir(t T) {
 }
 
 func testCLISchemaDiff(t T, url string) {
-	err := initCLI()
-
-	require.NoError(t, err)
 	t.dropTables("users")
-	cmd := exec.Command("go", "run", "ariga.io/atlas/cmd/atlas",
+	cmd := exec.Command(execPath(t),
 		"schema",
 		"diff",
 		"--from",
@@ -597,7 +596,11 @@ func testExecutor(t T) {
 	petsT.ForeignKeys = []*schema.ForeignKey{
 		{Symbol: "owner_id", Table: petsT, Columns: petsT.Columns[1:], RefTable: usersT, RefColumns: usersT.Columns[:1]},
 	}
-
+	if tt, ok := t.(interface {
+		pets(_, _ *schema.Table) *schema.Table
+	}); ok {
+		petsT = tt.pets(usersT, postsT)
+	}
 	t.dropTables(petsT.Name, postsT.Name, usersT.Name)
 	t.Cleanup(func() {
 		t.revisionsStorage().(*rrw).clean()
@@ -687,14 +690,21 @@ func (r *rrw) clean() {
 	*r = []*migrate.Revision{}
 }
 
-var _ migrate.RevisionReadWriter = (*rrw)(nil)
+var (
+	buildFlags []string
+	_          migrate.RevisionReadWriter = (*rrw)(nil)
+	buildOnce  sync.Once
+)
 
-func buildCmd(t *testing.T) (string, error) {
-	td := t.TempDir()
-	if b, err := exec.Command("go", "build", "-o", td, "ariga.io/atlas/cmd/atlas").CombinedOutput(); err != nil {
-		return "", fmt.Errorf("%w: %s", err, b)
-	}
-	return filepath.Join(td, "atlas"), nil
+func cliPath(t testing.TB) string {
+	path := filepath.Join(os.TempDir(), "atlas")
+	buildOnce.Do(func() {
+		args := append([]string{"build"}, buildFlags...)
+		args = append(args, "-o", path, "ariga.io/atlas/cmd/atlas")
+		out, err := exec.Command("go", args...).CombinedOutput()
+		require.NoError(t, err, string(out))
+	})
+	return path
 }
 
 func evalBytes(b []byte, v any, ev schemahcl.Evaluator) error {
