@@ -8,7 +8,6 @@ import (
 	"context"
 	"database/sql"
 	"fmt"
-	"log"
 	"os/exec"
 	"strings"
 	"sync"
@@ -50,14 +49,11 @@ func myRun(t *testing.T, fn func(*myTest)) {
 					tt.version = version
 					tt.rrw = &rrw{}
 					tt.db, err = sql.Open("mysql", fmt.Sprintf("root:pass@tcp(localhost:%d)/test?parseTime=True", tt.port))
-					if err != nil {
-						log.Fatalln(err)
-					}
-					dbs = append(dbs, tt.db) // close connection after all tests have been run
+					require.NoError(t, err)
+					// Close connection after all tests have been run.
+					dbs = append(dbs, tt.db)
 					tt.drv, err = mysql.Open(tt.db)
-					if err != nil {
-						log.Fatalln(err)
-					}
+					require.NoError(t, err)
 				})
 				tt := &myTest{T: t, db: tt.db, drv: tt.drv, version: version, port: tt.port, rrw: tt.rrw}
 				fn(tt)
@@ -556,7 +552,9 @@ func TestMySQL_Snapshot(t *testing.T) {
 		_, err = drv.(migrate.Snapshoter).Snapshot(context.Background())
 		require.ErrorAs(t, err, new(*migrate.NotCleanError))
 
-		r, err := t.driver().InspectRealm(context.Background(), nil)
+		r, err := t.driver().InspectRealm(context.Background(), &schema.InspectRealmOption{
+			Mode: ^schema.InspectViews,
+		})
 		require.NoError(t, err)
 		restore, err := t.driver().(migrate.Snapshoter).Snapshot(context.Background())
 		require.NoError(t, err) // connected to test schema
@@ -568,7 +566,9 @@ func TestMySQL_Snapshot(t *testing.T) {
 			t.dropTables("my_table")
 		})
 		require.NoError(t, restore(context.Background()))
-		r1, err := t.driver().InspectRealm(context.Background(), nil)
+		r1, err := t.driver().InspectRealm(context.Background(), &schema.InspectRealmOption{
+			Mode: ^schema.InspectViews,
+		})
 		require.NoError(t, err)
 		diff, err := t.driver().RealmDiff(r1, r)
 		require.NoError(t, err)
@@ -595,7 +595,7 @@ func TestMySQL_CLI_MigrateApplyLock(t *testing.T) {
 			go func(i int) {
 				defer wg.Done()
 				out, err := exec.Command(
-					"go", "run", "ariga.io/atlas/cmd/atlas",
+					execPath(t),
 					"migrate", "apply",
 					"--dir", "file://testdata/migrations/mysqlock",
 					"--url", t.url("mysqlock"),
@@ -750,7 +750,9 @@ schema "second" {
 }
 `
 		t.applyRealmHcl(wa)
-		realm, err = t.drv.InspectRealm(context.Background(), &schema.InspectRealmOption{})
+		realm, err = t.drv.InspectRealm(context.Background(), &schema.InspectRealmOption{
+			Mode: ^schema.InspectViews,
+		})
 		require.NoError(t, err)
 		_, ok := realm.Schema("test")
 		require.True(t, ok)
@@ -819,7 +821,10 @@ schema "users" {
 		hcl, err := mysql.MarshalHCL(realm)
 		require.NoError(t, err)
 		t.applyRealmHcl(string(hcl) + "\n" + expected)
-		realm, err = t.drv.InspectRealm(context.Background(), &schema.InspectRealmOption{Schemas: []string{"users", "financial"}})
+		realm, err = t.drv.InspectRealm(context.Background(), &schema.InspectRealmOption{
+			Mode:    ^schema.InspectViews,
+			Schemas: []string{"users", "financial"},
+		})
 		require.NoError(t, err)
 		actual, err := mysql.MarshalHCL(realm)
 		require.NoError(t, err)
@@ -1482,6 +1487,7 @@ func (t *myTest) quoted(s string) string {
 func (t *myTest) loadRealm() *schema.Realm {
 	r, err := t.drv.InspectRealm(context.Background(), &schema.InspectRealmOption{
 		Schemas: []string{"test"},
+		Mode:    ^schema.InspectViews,
 	})
 	require.NoError(t, err)
 	return r
