@@ -23,6 +23,7 @@ import (
 	"ariga.io/atlas/sql/mysql"
 	"ariga.io/atlas/sql/postgres"
 	"ariga.io/atlas/sql/schema"
+	"ariga.io/atlas/sql/sqlclient"
 	"ariga.io/atlas/sql/sqlite"
 
 	"github.com/pkg/diff"
@@ -102,7 +103,15 @@ func (t *myTest) setupScript(env *testscript.Env) error {
 	if err := replaceDBURL(env, t.url("")); err != nil {
 		return err
 	}
-	return setupScript(t.T, env, t.db, "DROP SCHEMA IF EXISTS %s")
+	return setupScript(env, t.db,
+		"CREATE SCHEMA IF NOT EXISTS %s",
+		"DROP SCHEMA IF EXISTS %s",
+		func(tt testing.TB, schema string) migrate.Driver {
+			dev, err := sqlclient.Open(context.Background(), fmt.Sprintf("mysql://root:pass@localhost:%d/%s", t.port, schema))
+			require.NoError(tt, err)
+			tt.Cleanup(func() { require.NoError(tt, dev.Close()) })
+			return dev.Driver
+		})
 }
 
 func replaceDBURL(env *testscript.Env, url string) error {
@@ -121,34 +130,55 @@ func (t *pgTest) setupScript(env *testscript.Env) error {
 	if err := replaceDBURL(env, u); err != nil {
 		return err
 	}
-	return setupScript(t.T, env, t.db, "DROP SCHEMA IF EXISTS %s CASCADE")
+	return setupScript(env, t.db,
+		"CREATE SCHEMA IF NOT EXISTS %s",
+		"DROP SCHEMA IF EXISTS %s CASCADE",
+		func(tt testing.TB, schema string) migrate.Driver {
+			dev, err := sqlclient.Open(context.Background(), fmt.Sprintf("postgres://postgres:pass@localhost:%d/test?search_path=%s&sslmode=disable", t.port, schema))
+			require.NoError(tt, err)
+			tt.Cleanup(func() { require.NoError(tt, dev.Close()) })
+			return dev.Driver
+		})
 }
 
-func setupScript(t *testing.T, env *testscript.Env, db *sql.DB, dropCmd string) error {
+func setupScript(env *testscript.Env, db *sql.DB, createCmd, dropCmd string, open func(testing.TB, string) migrate.Driver) error {
 	ctx := context.Background()
 	conn, err := db.Conn(ctx)
 	if err != nil {
 		return err
 	}
 	name := strings.ReplaceAll(filepath.Base(env.WorkDir), "-", "_")
+	devname := fmt.Sprintf("%s_dev", name)
 	env.Setenv("db", name)
-	if _, err := conn.ExecContext(ctx, fmt.Sprintf("CREATE SCHEMA IF NOT EXISTS %s", name)); err != nil {
+	env.Setenv("dev", devname)
+	if _, err := conn.ExecContext(ctx, fmt.Sprintf(createCmd, name)); err != nil {
 		return err
 	}
-	env.Defer(func() {
+	if _, err := conn.ExecContext(ctx, fmt.Sprintf(createCmd, devname)); err != nil {
+		return err
+	}
+	// Env has a T() method that returns the *testing.T.
+	// We need to cast it to the testing.TB interface
+	// to be able to use the Cleanup method.
+	tt := env.T().(testing.TB)
+	tt.Cleanup(func() {
 		if _, err := conn.ExecContext(ctx, fmt.Sprintf(dropCmd, name)); err != nil {
-			t.Fatal(err)
+			tt.Fatal(err)
+		}
+		if _, err := conn.ExecContext(ctx, fmt.Sprintf(dropCmd, devname)); err != nil {
+			tt.Fatal(err)
 		}
 		if err := conn.Close(); err != nil {
-			t.Fatal(err)
+			tt.Fatal(err)
 		}
 	})
+	// Dev-driver per testscript schema to allow concurrent tests.
+	env.Values["drv"] = open(tt, name)
+	env.Values["dev"] = open(tt, devname)
 	// Store the testscript.T for later use.
 	// See "only" function below.
 	env.Values[keyT] = env.T()
-	if err := setupCLITest(t, env); err != nil {
-		return err
-	}
+	env.Setenv(atlasPathKey, cliPath(tt))
 	return nil
 }
 
@@ -173,9 +203,7 @@ func (t *liteTest) setupScript(env *testscript.Env) error {
 	// environment as tests run in parallel.
 	env.Values[keyDB] = db
 	env.Values[keyDrv] = drv
-	if err := setupCLITest(t.T, env); err != nil {
-		return err
-	}
+	env.Setenv(atlasPathKey, cliPath(t.T))
 	// Set the workdir in the test atlas.hcl file.
 	projectFile := filepath.Join(env.WorkDir, "atlas.hcl")
 	if b, err := os.ReadFile(projectFile); err == nil {
@@ -186,15 +214,6 @@ func (t *liteTest) setupScript(env *testscript.Env) error {
 	return nil
 }
 
-func setupCLITest(t *testing.T, env *testscript.Env) error {
-	path, err := buildCmd(t)
-	if err != nil {
-		return err
-	}
-	env.Setenv(atlasPathKey, path)
-	return nil
-}
-
 // cmdOnly executes only tests that their driver version matches the given pattern.
 // For example, "only 8" or "only 8 maria*"
 func cmdOnly(ts *testscript.TestScript, neg bool, args []string) {
@@ -309,7 +328,7 @@ func cmdCmpShow(ts *testscript.TestScript, args []string, show func(schema, name
 func (t *myTest) cmdCmpHCL(ts *testscript.TestScript, _ bool, args []string) {
 	r := strings.NewReplacer("$charset", ts.Getenv("charset"), "$collate", ts.Getenv("collate"), "$db", ts.Getenv("db"))
 	cmdCmpHCL(ts, args, func(name string) (string, error) {
-		s, err := t.drv.InspectSchema(context.Background(), name, nil)
+		s, err := ts.Value("drv").(migrate.Driver).InspectSchema(context.Background(), name, nil)
 		ts.Check(err)
 		buf, err := mysql.MarshalHCL(s)
 		require.NoError(t, err)
@@ -321,7 +340,7 @@ func (t *myTest) cmdCmpHCL(ts *testscript.TestScript, _ bool, args []string) {
 
 func (t *pgTest) cmdCmpHCL(ts *testscript.TestScript, _ bool, args []string) {
 	cmdCmpHCL(ts, args, func(name string) (string, error) {
-		s, err := t.drv.InspectSchema(context.Background(), name, nil)
+		s, err := ts.Value("drv").(migrate.Driver).InspectSchema(context.Background(), name, nil)
 		ts.Check(err)
 		buf, err := postgres.MarshalHCL(s)
 		require.NoError(t, err)
@@ -558,7 +577,7 @@ func (t *myTest) cmdSynced(ts *testscript.TestScript, neg bool, args []string) {
 }
 
 func (t *myTest) cmdApply(ts *testscript.TestScript, neg bool, args []string) {
-	cmdApply(ts, neg, args, t.drv.ApplyChanges, t.hclDiff)
+	cmdApply(ts, neg, args, ts.Value("drv").(migrate.Driver).ApplyChanges, t.hclDiff)
 }
 
 func (t *myTest) hclDiff(ts *testscript.TestScript, name string) ([]schema.Change, error) {
@@ -566,18 +585,19 @@ func (t *myTest) hclDiff(ts *testscript.TestScript, name string) ([]schema.Chang
 		desired = &schema.Schema{}
 		f       = ts.ReadFile(name)
 		ctx     = context.Background()
+		drv     = ts.Value("drv").(migrate.Driver)
 		r       = strings.NewReplacer("$charset", ts.Getenv("charset"), "$collate", ts.Getenv("collate"), "$db", ts.Getenv("db"))
 	)
 	ts.Check(mysql.EvalHCLBytes([]byte(r.Replace(f)), desired, nil))
-	current, err := t.drv.InspectSchema(ctx, desired.Name, nil)
+	current, err := drv.InspectSchema(ctx, desired.Name, nil)
 	ts.Check(err)
-	desired, err = t.drv.(schema.Normalizer).NormalizeSchema(ctx, desired)
+	desired, err = ts.Value("dev").(schema.Normalizer).NormalizeSchema(ctx, desired)
 	// Normalization and diffing errors should
 	// be returned to the caller.
 	if err != nil {
 		return nil, err
 	}
-	changes, err := t.drv.SchemaDiff(current, desired)
+	changes, err := drv.SchemaDiff(current, desired)
 	if err != nil {
 		return nil, err
 	}
@@ -589,25 +609,26 @@ func (t *pgTest) cmdSynced(ts *testscript.TestScript, neg bool, args []string) {
 }
 
 func (t *pgTest) cmdApply(ts *testscript.TestScript, neg bool, args []string) {
-	cmdApply(ts, neg, args, t.drv.ApplyChanges, t.hclDiff)
+	cmdApply(ts, neg, args, ts.Value("drv").(migrate.Driver).ApplyChanges, t.hclDiff)
 }
 
 func (t *pgTest) hclDiff(ts *testscript.TestScript, name string) ([]schema.Change, error) {
 	var (
 		desired = &schema.Schema{}
 		ctx     = context.Background()
+		drv     = ts.Value("drv").(migrate.Driver)
 		f       = strings.ReplaceAll(ts.ReadFile(name), "$db", ts.Getenv("db"))
 	)
 	ts.Check(postgres.EvalHCLBytes([]byte(f), desired, nil))
-	current, err := t.drv.InspectSchema(ctx, desired.Name, nil)
+	current, err := drv.InspectSchema(ctx, desired.Name, nil)
 	ts.Check(err)
-	desired, err = t.drv.(schema.Normalizer).NormalizeSchema(ctx, desired)
+	desired, err = ts.Value("dev").(schema.Normalizer).NormalizeSchema(ctx, desired)
 	// Normalization and diffing errors should
 	// be returned to the caller.
 	if err != nil {
 		return nil, err
 	}
-	changes, err := t.drv.SchemaDiff(current, desired)
+	changes, err := drv.SchemaDiff(current, desired)
 	if err != nil {
 		return nil, err
 	}
