@@ -72,7 +72,7 @@ func (s *State) setInputVals(ctx *hcl.EvalContext, body hcl.Body, input map[stri
 	return nil
 }
 
-// evalReferences evaluates data blocks.
+// evalReferences evaluates local and data blocks.
 func (s *State) evalReferences(ctx *hcl.EvalContext, body *hclsyntax.Body) error {
 	type node struct {
 		addr  [3]string
@@ -141,7 +141,7 @@ func (s *State) evalReferences(ctx *hcl.EvalContext, body *hclsyntax.Body) error
 		}
 	}
 	var (
-		visit    func(n *node) error
+		visit    func(*node) error
 		visited  = make(map[*node]bool)
 		progress = make(map[*node]bool)
 	)
@@ -160,7 +160,7 @@ func (s *State) evalReferences(ctx *hcl.EvalContext, body *hclsyntax.Body) error
 		for _, e := range n.edges() {
 			var addr [3]string
 			switch root := e.RootName(); {
-			case root == RefLocal && len(e) == 2:
+			case root == RefLocal && len(e) > 1:
 				addr = [3]string{RefLocal, e[1].(hcl.TraverseAttr).Name, ""}
 			case root == RefData && len(e) > 2:
 				addr = [3]string{RefData, e[1].(hcl.TraverseAttr).Name, e[2].(hcl.TraverseAttr).Name}
@@ -212,7 +212,25 @@ func (s *State) evalReferences(ctx *hcl.EvalContext, body *hclsyntax.Body) error
 			return err
 		}
 	}
+	dataref := dataRefs(body)
 	for _, n := range nodes {
+		// Evaluate data sources only if they were referenced by other top-level
+		// blocks/attributes or if they reference other evaluated data sources.
+		if n.addr[0] == BlockData {
+			exists := func() bool {
+				for _, r := range dataref {
+					t, ok1 := r[1].(hcl.TraverseAttr)
+					l, ok2 := r[2].(hcl.TraverseAttr)
+					if ok1 && ok2 && t.Name == n.addr[1] && l.Name == n.addr[2] {
+						return true
+					}
+				}
+				return false
+			}()
+			if !exists {
+				continue
+			}
+		}
 		if err := visit(n); err != nil {
 			return err
 		}
@@ -273,6 +291,10 @@ func blockVars(blocks hclsyntax.Blocks, parentAddr string, defs *blockDef) (map[
 			}
 			self := addr(parentAddr, name, blkName, qualifier)
 			attrs["__ref"] = cty.StringVal(self)
+			// Skip naming blocks with "name" attribute.
+			if _, ok := blk.Body.Attributes["name"]; !ok {
+				attrs["name"] = cty.StringVal(blkName)
+			}
 			varMap, err := blockVars(blk.Body.Blocks, self, def)
 			if err != nil {
 				return nil, err
@@ -391,43 +413,6 @@ type Variables struct {
 	Data       map[string]map[string]bool
 }
 
-// DynamicRefs returns the references to dynamic variables.
-// i.e, variables, locals, and data sources.
-func DynamicRefs(b *hclsyntax.Body) *Variables {
-	var (
-		vars = &Variables{
-			Var:   make(map[string]bool),
-			Local: make(map[string]bool),
-			Data:  make(map[string]map[string]bool),
-		}
-		visit func(b *hclsyntax.Body, vars *Variables)
-	)
-	visit = func(b *hclsyntax.Body, vars *Variables) {
-		for _, a := range b.Attributes {
-			for _, v := range hclsyntax.Variables(a.Expr) {
-				switch {
-				case len(v) == 0:
-				case v.RootName() == RefVar && len(v) > 1:
-					vars.Var[v[1].(hcl.TraverseAttr).Name] = true
-				case v.RootName() == RefLocal && len(v) > 1:
-					vars.Local[v[1].(hcl.TraverseAttr).Name] = true
-				case v.RootName() == RefData && len(v) > 2:
-					n1, n2 := v[1].(hcl.TraverseAttr).Name, v[2].(hcl.TraverseAttr).Name
-					if _, ok := vars.Data[n1]; !ok {
-						vars.Data[n1] = make(map[string]bool)
-					}
-					vars.Data[n1][n2] = true
-				}
-			}
-		}
-		for _, b := range b.Blocks {
-			visit(b.Body, vars)
-		}
-	}
-	visit(b, vars)
-	return vars
-}
-
 // defRegistry returns a tree of blockDef structs representing the schema of the
 // blocks in the *hclsyntax.Body. The returned fields and children of each type
 // are an intersection of all existing blocks of the same type.
@@ -475,6 +460,7 @@ func (t *blockDef) asCty() cty.Type {
 	for attr := range t.fields {
 		f[attr] = ctyNilType
 	}
+	f["name"] = cty.String
 	f["__ref"] = cty.String
 	for _, c := range t.children {
 		f[c.name] = c.asCty()
@@ -499,11 +485,28 @@ func extractDef(blk *hclsyntax.Block, parent *blockDef) *blockDef {
 }
 
 func bodyVars(b *hclsyntax.Body) (vars []hcl.Traversal) {
-	for _, attr := range b.Attributes {
-		vars = append(vars, hclsyntax.Variables(attr.Expr)...)
+	for _, a := range b.Attributes {
+		vars = append(vars, hclsyntax.Variables(a.Expr)...)
 	}
 	for _, b := range b.Blocks {
 		vars = append(vars, bodyVars(b.Body)...)
 	}
 	return
 }
+
+// dataRefs returns all data source referenced in the body.
+func dataRefs(b *hclsyntax.Body) (refs []hcl.Traversal) {
+	for _, a := range b.Attributes {
+		for _, v := range hclsyntax.Variables(a.Expr) {
+			if v.RootName() == RefData {
+				refs = append(refs, v)
+			}
+		}
+	}
+	for _, b := range b.Blocks {
+		if b.Type != BlockData {
+			refs = append(refs, dataRefs(b.Body)...)
+		}
+	}
+	return
+}
