@@ -25,6 +25,10 @@ b  = true
 s  = "hello, world"
 sl = ["hello", "world"]
 bl = [true, false]
+hd = <<-EOT
+  hello
+  world
+EOT
 `
 	var test struct {
 		Int        int      `spec:"i"`
@@ -32,6 +36,7 @@ bl = [true, false]
 		Str        string   `spec:"s"`
 		StringList []string `spec:"sl"`
 		BoolList   []bool   `spec:"bl"`
+		Heredoc    string   `spec:"hd"`
 	}
 	err := New().EvalBytes([]byte(f), &test, nil)
 	require.NoError(t, err)
@@ -40,6 +45,9 @@ bl = [true, false]
 	require.EqualValues(t, "hello, world", test.Str)
 	require.EqualValues(t, []string{"hello", "world"}, test.StringList)
 	require.EqualValues(t, []bool{true, false}, test.BoolList)
+	require.EqualValues(t, "hello\nworld\n", test.Heredoc)
+	// Heredoc needs to be explicitly formatted this way.
+	test.Heredoc = "<<-EOT\n  hello\n  world\nEOT"
 	marshal, err := Marshal(&test)
 	require.NoError(t, err)
 	require.EqualValues(t, f, string(marshal))
@@ -543,6 +551,9 @@ locals {
   c = "local-c-ref-data-a: ${data.text.a.output}"
   d = "local-d"
   host = "atlasgo.io"
+  obj = {
+    k = "obj-v"
+  }
 }
 
 data "sql" "tenants" {
@@ -578,6 +589,9 @@ data "remote_dir" "migrations" {
   host = atlas.auth.host
 }
 
+data "text" "obj" {
+  value = local.obj.k
+}
 vs = [
   local.a,
   local.b,
@@ -586,6 +600,7 @@ vs = [
   data.text.a.output,
   data.text.b.output,
   data.remote_dir.migrations.url,
+  data.text.obj.output,
 ]
 `)
 	)
@@ -598,6 +613,7 @@ vs = [
 		"data-text-a-ref-data-sql-tenants: data-sql-tenants",
 		"data-text-b-ref-local-d: local-d",
 		"atlas://atlasgo.io/ent/migrations",
+		"obj-v",
 	}, doc.Values)
 
 	b = []byte(`locals { a = local.a }`)
@@ -623,3 +639,62 @@ locals {
 `)
 	require.Error(t, New(opts...).EvalBytes(b, &doc, nil), `cyclic reference to "data.text.a"`)
 }
+
+func TestSkippedDataSrc(t *testing.T) {
+	var (
+		opts = []Option{
+			WithDataSource("dynamic", func(ctx *hcl.EvalContext, b *hclsyntax.Block) (cty.Value, error) {
+				attrs, diags := b.Body.JustAttributes()
+				if diags.HasErrors() {
+					return cty.NilVal, diags
+				}
+				s, diags := attrs["skip"].Expr.Value(ctx)
+				if diags.HasErrors() {
+					return cty.NilVal, diags
+				}
+				if s.True() {
+					return cty.NilVal, fmt.Errorf("data source should be skipped, but was called with %q", b.Labels)
+				}
+				v, diags := attrs["v"].Expr.Value(ctx)
+				if diags.HasErrors() {
+					return cty.NilVal, diags
+				}
+				return cty.ObjectVal(map[string]cty.Value{
+					"v": v,
+				}), nil
+			}),
+		}
+		v struct {
+			V2 string `spec:"v2"`
+			V3 string `spec:"v3"`
+		}
+		b = []byte(`
+data "dynamic" "skipped1" {
+  v = "value is irrelevant"
+  // This attribute has no meaning, besides indicating
+  // to the test that the data source should be skipped.
+  skip = true
+}
+data "dynamic" "skipped2" {
+  v = data.dynamic.skipped1.v
+  skip = true
+}
+data "dynamic" "evaluated1" {
+  v = "v1"
+  skip = false
+}
+data "dynamic" "evaluated2" {
+  v = "v2"
+  skip = false
+}
+data "dynamic" "evaluated3" {
+  v = data.dynamic.evaluated1.v
+  skip = false
+}
+
+v2 = data.dynamic.evaluated2.v
+v3 = data.dynamic.evaluated3.v
+`)
+	)
+	require.NoError(t, New(opts...).EvalBytes(b, &v, nil))
+}
