@@ -27,10 +27,12 @@ func TestPlanChanges(t *testing.T) {
 	}{
 		{
 			changes: []schema.Change{
-				&schema.AddSchema{S: schema.New("public")},
+				&schema.AddSchema{S: schema.New("public").SetComment("public schema")},
 				&schema.AddSchema{S: schema.New("test"), Extra: []schema.Clause{&schema.IfNotExists{}}},
 				&schema.DropSchema{S: schema.New("test"), Extra: []schema.Clause{&schema.IfExists{}}},
 				&schema.DropSchema{S: schema.New("test"), Extra: []schema.Clause{}},
+				&schema.ModifySchema{S: schema.New("modify1").SetComment("comment"), Changes: schema.Changes{&schema.AddAttr{A: &schema.Comment{Text: "comment"}}}},
+				&schema.ModifySchema{S: schema.New("modify2").SetComment("comment"), Changes: schema.Changes{&schema.ModifyAttr{From: &schema.Comment{Text: "old"}, To: &schema.Comment{Text: "new"}}}},
 			},
 			wantPlan: &migrate.Plan{
 				Reversible:    false,
@@ -40,6 +42,10 @@ func TestPlanChanges(t *testing.T) {
 						Cmd:     `CREATE SCHEMA IF NOT EXISTS "public"`,
 						Reverse: `DROP SCHEMA "public" CASCADE`,
 					},
+					{
+						Cmd:     `COMMENT ON SCHEMA "public" IS 'public schema'`,
+						Reverse: `COMMENT ON SCHEMA "public" IS ''`,
+					},
 					{
 						Cmd:     `CREATE SCHEMA IF NOT EXISTS "test"`,
 						Reverse: `DROP SCHEMA "test" CASCADE`,
@@ -50,6 +56,14 @@ func TestPlanChanges(t *testing.T) {
 					{
 						Cmd: `DROP SCHEMA "test" CASCADE`,
 					},
+					{
+						Cmd:     `COMMENT ON SCHEMA "modify1" IS 'comment'`,
+						Reverse: `COMMENT ON SCHEMA "modify1" IS ''`,
+					},
+					{
+						Cmd:     `COMMENT ON SCHEMA "modify2" IS 'new'`,
+						Reverse: `COMMENT ON SCHEMA "modify2" IS 'old'`,
+					},
 				},
 			},
 		},
@@ -187,6 +201,13 @@ func TestPlanChanges(t *testing.T) {
 		},
 		{
 			changes: []schema.Change{
+				&schema.AddObject{
+					O: &schema.EnumType{
+						T:      "direction",
+						Values: []string{"NORTH", "SOUTH"},
+						Schema: schema.New("public"),
+					},
+				},
 				&schema.AddTable{
 					T: &schema.Table{
 						Name: "posts",
@@ -345,6 +366,35 @@ func TestPlanChanges(t *testing.T) {
 				},
 			},
 		},
+		{
+			changes: []schema.Change{
+				&schema.DropObject{O: &schema.EnumType{T: "status", Values: []string{"on", "off"}, Schema: schema.New("public")}},
+				&schema.DropTable{T: schema.NewTable("logs").
+					AddColumns(
+						schema.NewColumn("status").SetType(&schema.EnumType{T: "status", Values: []string{"on", "off"}, Schema: schema.New("public")}),
+					),
+				},
+				&schema.DropObject{O: &schema.EnumType{T: "state", Values: []string{"on", "off"}, Schema: schema.New("public")}},
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     `DROP TABLE "logs"`,
+						Reverse: `CREATE TABLE "logs" ("status" "public"."status" NOT NULL)`,
+					},
+					{
+						Cmd:     `DROP TYPE "public"."status"`,
+						Reverse: `CREATE TYPE "public"."status" AS ENUM ('on', 'off')`,
+					},
+					{
+						Cmd:     `DROP TYPE "public"."state"`,
+						Reverse: `CREATE TYPE "public"."state" AS ENUM ('on', 'off')`,
+					},
+				},
+			},
+		},
 		{
 			changes: []schema.Change{
 				&schema.DropTable{
@@ -733,11 +783,25 @@ func TestPlanChanges(t *testing.T) {
 		},
 		{
 			changes: []schema.Change{
+				&schema.AddObject{
+					O: &schema.EnumType{
+						T:      "state",
+						Values: []string{"on", "off"},
+						Schema: schema.New("public"),
+					},
+				},
+				&schema.AddObject{
+					O: &schema.EnumType{
+						T:      "status",
+						Values: []string{"a", "b"},
+						Schema: schema.New("test"),
+					},
+				},
 				func() schema.Change {
 					users := schema.NewTable("users").
 						SetSchema(schema.New("public")).
 						AddColumns(
-							schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off")),
+							schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off"), schema.EnumSchema(schema.New("public"))),
 							schema.NewEnumColumn("status", schema.EnumName("status"), schema.EnumValues("a", "b"), schema.EnumSchema(schema.New("test"))),
 						)
 					return &schema.ModifyTable{
@@ -754,23 +818,15 @@ func TestPlanChanges(t *testing.T) {
 								Change: schema.ChangeType,
 							},
 							&schema.DropColumn{
-								C: schema.NewEnumColumn("dc1", schema.EnumName("de"), schema.EnumValues("on")),
+								C: schema.NewEnumColumn("dc1", schema.EnumName("de"), schema.EnumValues("on"), schema.EnumSchema(schema.New("public"))),
 							},
 							&schema.DropColumn{
-								C: schema.NewEnumColumn("dc2", schema.EnumName("de"), schema.EnumValues("on")),
+								C: schema.NewEnumColumn("dc2", schema.EnumName("de"), schema.EnumValues("on"), schema.EnumSchema(schema.New("public"))),
 							},
 						},
 					}
 				}(),
 			},
-			mock: func(m mock) {
-				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e' AND n.nspname = $2 ")).
-					WithArgs("state", "public").
-					WillReturnRows(sqlmock.NewRows([]string{"name"}))
-				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e' AND n.nspname = $2 ")).
-					WithArgs("status", "test").
-					WillReturnRows(sqlmock.NewRows([]string{"name"}))
-			},
 			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
@@ -784,22 +840,28 @@ func TestPlanChanges(t *testing.T) {
 		},
 		{
 			changes: []schema.Change{
+				&schema.DropObject{
+					O: &schema.EnumType{
+						T:      "state",
+						Values: []string{"on", "off"},
+						Schema: schema.New("public"),
+					},
+				},
 				&schema.DropTable{
 					T: schema.NewTable("users").
 						SetSchema(schema.New("public")).
 						AddColumns(
-							schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off")),
-							schema.NewEnumColumn("status", schema.EnumName("status"), schema.EnumValues("on", "off")),
+							schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off"), schema.EnumSchema(schema.New("public"))),
+							schema.NewEnumColumn("status", schema.EnumName("status"), schema.EnumValues("on", "off"), schema.EnumSchema(schema.New("public"))),
 						),
 				},
-			},
-			mock: func(m mock) {
-				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e' AND n.nspname = $2 ")).
-					WithArgs("state", "public").
-					WillReturnRows(sqlmock.NewRows([]string{"name"}))
-				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e' AND n.nspname = $2 ")).
-					WithArgs("status", "public").
-					WillReturnRows(sqlmock.NewRows([]string{"name"}))
+				&schema.DropObject{
+					O: &schema.EnumType{
+						T:      "status",
+						Values: []string{"on", "off"},
+						Schema: schema.New("public"),
+					},
+				},
 			},
 			wantPlan: &migrate.Plan{
 				Reversible:    true,
@@ -817,19 +879,11 @@ func TestPlanChanges(t *testing.T) {
 					T: schema.NewTable("users").
 						SetSchema(schema.New("public")).
 						AddColumns(
-							schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off")),
-							schema.NewEnumColumn("status", schema.EnumName("status"), schema.EnumValues("on", "off")),
+							schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off"), schema.EnumSchema(schema.New("public"))),
+							schema.NewEnumColumn("status", schema.EnumName("status"), schema.EnumValues("on", "off"), schema.EnumSchema(schema.New("public"))),
 						),
 				},
 			},
-			mock: func(m mock) {
-				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e' AND n.nspname = $2 ")).
-					WithArgs("state", "public").
-					WillReturnRows(sqlmock.NewRows([]string{"name"}))
-				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e' AND n.nspname = $2 ")).
-					WithArgs("status", "public").
-					WillReturnRows(sqlmock.NewRows([]string{"name"}))
-			},
 			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
@@ -846,14 +900,21 @@ func TestPlanChanges(t *testing.T) {
 					AddTables(
 						schema.NewTable("t1").
 							AddColumns(
-								schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off")),
+								schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off"), schema.EnumSchema(schema.New("public"))),
 							),
 						schema.NewTable("t2").
 							AddColumns(
-								schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off")),
+								schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off"), schema.EnumSchema(schema.New("public"))),
 							),
 					)
 				return []schema.Change{
+					&schema.DropObject{
+						O: &schema.EnumType{
+							T:      "state",
+							Values: []string{"on", "off"},
+							Schema: schema.New("public"),
+						},
+					},
 					&schema.DropTable{
 						T: s.Tables[0],
 					},
@@ -862,14 +923,6 @@ func TestPlanChanges(t *testing.T) {
 					},
 				}
 			}(),
-			mock: func(m mock) {
-				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e' AND n.nspname = $2 ")).
-					WithArgs("state", "public").
-					WillReturnRows(sqlmock.NewRows([]string{"name"}).AddRow("state"))
-				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e' AND n.nspname = $2 ")).
-					WithArgs("state", "public").
-					WillReturnRows(sqlmock.NewRows([]string{"name"}).AddRow("state"))
-			},
 			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
@@ -882,29 +935,27 @@ func TestPlanChanges(t *testing.T) {
 						Cmd:     `DROP TABLE "public"."t2"`,
 						Reverse: `CREATE TABLE "public"."t2" ("state" "public"."state" NOT NULL)`,
 					},
-					{Cmd: `DROP TYPE "public"."state"`, Reverse: `CREATE TYPE "public"."state" AS ENUM ('on', 'off')`},
+					{
+						Cmd:     `DROP TYPE "public"."state"`,
+						Reverse: `CREATE TYPE "public"."state" AS ENUM ('on', 'off')`,
+					},
 				},
 			},
 		},
 		{
 			changes: []schema.Change{
-				func() schema.Change {
-					users := schema.NewTable("users").
-						SetSchema(schema.New("public")).
-						AddColumns(
-							schema.NewEnumColumn("state", schema.EnumName("state"), schema.EnumValues("on", "off")),
-						)
-					return &schema.ModifyTable{
-						T: users,
-						Changes: []schema.Change{
-							&schema.ModifyColumn{
-								From:   users.Columns[0],
-								To:     &schema.Column{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off", "unknown"}}}},
-								Change: schema.ChangeType,
-							},
-						},
-					}
-				}(),
+				&schema.ModifyObject{
+					From: &schema.EnumType{
+						T:      "state",
+						Values: []string{"on", "off"},
+						Schema: schema.New("public"),
+					},
+					To: &schema.EnumType{
+						T:      "state",
+						Values: []string{"on", "off", "unknown"},
+						Schema: schema.New("public"),
+					},
+				},
 			},
 			wantPlan: &migrate.Plan{
 				Reversible:    false,
@@ -917,6 +968,18 @@ func TestPlanChanges(t *testing.T) {
 		// Modify column type and drop comment.
 		{
 			changes: []schema.Change{
+				&schema.ModifyObject{
+					From: &schema.EnumType{
+						T:      "state",
+						Values: []string{"on", "off"},
+						Schema: schema.New("public"),
+					},
+					To: &schema.EnumType{
+						T:      "state",
+						Values: []string{"on", "off", "unknown"},
+						Schema: schema.New("public"),
+					},
+				},
 				func() schema.Change {
 					users := schema.NewTable("users").
 						SetSchema(schema.New("public")).
@@ -929,7 +992,7 @@ func TestPlanChanges(t *testing.T) {
 							&schema.ModifyColumn{
 								From:   &schema.Column{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off"}}}, Attrs: []schema.Attr{&schema.Comment{Text: "foo"}}},
 								To:     &schema.Column{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off", "unknown"}}}},
-								Change: schema.ChangeType | schema.ChangeComment,
+								Change: schema.ChangeComment,
 							},
 						},
 					}
@@ -944,37 +1007,6 @@ func TestPlanChanges(t *testing.T) {
 				},
 			},
 		},
-		// Modify column type and add comment.
-		{
-			changes: []schema.Change{
-				func() schema.Change {
-					users := &schema.Table{
-						Name: "users",
-						Columns: []*schema.Column{
-							{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off"}}}},
-						},
-					}
-					return &schema.ModifyTable{
-						T: users,
-						Changes: []schema.Change{
-							&schema.ModifyColumn{
-								From:   &schema.Column{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off"}}}},
-								To:     &schema.Column{Name: "state", Type: &schema.ColumnType{Type: &schema.EnumType{T: "state", Values: []string{"on", "off", "unknown"}}}, Attrs: []schema.Attr{&schema.Comment{Text: "foo"}}},
-								Change: schema.ChangeType | schema.ChangeComment,
-							},
-						},
-					}
-				}(),
-			},
-			wantPlan: &migrate.Plan{
-				Reversible:    false,
-				Transactional: true,
-				Changes: []*migrate.Change{
-					{Cmd: `ALTER TYPE "state" ADD VALUE 'unknown'`},
-					{Cmd: `COMMENT ON COLUMN "users" ."state" IS 'foo'`, Reverse: `COMMENT ON COLUMN "users" ."state" IS ''`},
-				},
-			},
-		},
 		// Modify column comment.
 		{
 			changes: []schema.Change{
@@ -1316,6 +1348,13 @@ func TestPlanChanges(t *testing.T) {
 		// Empty qualifier.
 		{
 			changes: []schema.Change{
+				&schema.AddObject{
+					O: &schema.EnumType{
+						T:      "enum",
+						Values: []string{"a"},
+						Schema: schema.New("ignored"),
+					},
+				},
 				&schema.AddTable{
 					T: schema.NewTable("posts").
 						SetSchema(schema.New("test1")).
@@ -1327,11 +1366,6 @@ func TestPlanChanges(t *testing.T) {
 			options: []migrate.PlanOption{
 				func(o *migrate.PlanOptions) { o.SchemaQualifier = new(string) },
 			},
-			mock: func(m mock) {
-				m.ExpectQuery(sqltest.Escape("SELECT * FROM pg_type t JOIN pg_namespace n on t.typnamespace = n.oid WHERE t.typname = $1 AND t.typtype = 'e'")).
-					WithArgs("enum").
-					WillReturnRows(sqlmock.NewRows([]string{"name"}))
-			},
 			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
