@@ -24,6 +24,7 @@ import (
 type (
 	doc struct {
 		Tables  []*sqlspec.Table  `spec:"table"`
+		Views   []*sqlspec.View   `spec:"view"`
 		Enums   []*Enum           `spec:"enum"`
 		Schemas []*sqlspec.Schema `spec:"schema"`
 	}
@@ -48,7 +49,10 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 		if err := hclState.Eval(p, &d, input); err != nil {
 			return err
 		}
-		if err := specutil.Scan(v, d.Schemas, d.Tables, convertTable); err != nil {
+		if err := specutil.Scan(v,
+			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views},
+			&specutil.ScanFuncs{Table: convertTable, View: convertView},
+		); err != nil {
 			return fmt.Errorf("specutil: failed converting to *schema.Realm: %w", err)
 		}
 		if len(d.Enums) > 0 {
@@ -65,7 +69,10 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 			return fmt.Errorf("specutil: expecting document to contain a single schema, got %d", len(d.Schemas))
 		}
 		r := &schema.Realm{}
-		if err := specutil.Scan(r, d.Schemas, d.Tables, convertTable); err != nil {
+		if err := specutil.Scan(r,
+			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views},
+			&specutil.ScanFuncs{Table: convertTable, View: convertView},
+		); err != nil {
 			return err
 		}
 		if err := convertEnums(d.Tables, d.Enums, r); err != nil {
@@ -91,6 +98,7 @@ func MarshalSpec(v any, marshaler schemahcl.Marshaler) ([]byte, error) {
 			return nil, fmt.Errorf("specutil: failed converting schema to spec: %w", err)
 		}
 		d.Tables = doc.Tables
+		d.Views = doc.Views
 		d.Schemas = doc.Schemas
 		d.Enums = doc.Enums
 	case *schema.Realm:
@@ -100,10 +108,14 @@ func MarshalSpec(v any, marshaler schemahcl.Marshaler) ([]byte, error) {
 				return nil, fmt.Errorf("specutil: failed converting schema to spec: %w", err)
 			}
 			d.Tables = append(d.Tables, doc.Tables...)
+			d.Views = append(d.Views, doc.Views...)
 			d.Schemas = append(d.Schemas, doc.Schemas...)
 			d.Enums = append(d.Enums, doc.Enums...)
 		}
-		if err := specutil.QualifyDuplicates(d.Tables); err != nil {
+		if err := specutil.QualifyTables(d.Tables); err != nil {
+			return nil, err
+		}
+		if err := specutil.QualifyViews(d.Views); err != nil {
 			return nil, err
 		}
 		if err := specutil.QualifyReferences(d.Tables, s); err != nil {
@@ -118,6 +130,8 @@ func MarshalSpec(v any, marshaler schemahcl.Marshaler) ([]byte, error) {
 var (
 	hclState = schemahcl.New(
 		schemahcl.WithTypes("table.column.type", TypeRegistry.Specs()),
+		schemahcl.WithTypes("view.column.type", TypeRegistry.Specs()),
+		schemahcl.WithScopedEnums("view.check_option", schema.ViewCheckOptionLocal, schema.ViewCheckOptionCascaded),
 		schemahcl.WithScopedEnums("table.index.type", IndexTypeBTree, IndexTypeBRIN, IndexTypeHash, IndexTypeGIN, IndexTypeGiST, "GiST", IndexTypeSPGiST, "SPGiST"),
 		schemahcl.WithScopedEnums("table.partition.type", PartitionTypeRange, PartitionTypeList, PartitionTypeHash),
 		schemahcl.WithScopedEnums("table.column.identity.generated", GeneratedTypeAlways, GeneratedTypeByDefault),
@@ -157,6 +171,17 @@ func convertTable(spec *sqlspec.Table, parent *schema.Schema) (*schema.Table, er
 	return t, nil
 }
 
+// convertView converts a sqlspec.View to a schema.View.
+func convertView(spec *sqlspec.View, parent *schema.Schema) (*schema.View, error) {
+	v, err := specutil.View(spec, parent, func(c *sqlspec.Column, _ *schema.View) (*schema.Column, error) {
+		return specutil.Column(c, convertColumnType)
+	})
+	if err != nil {
+		return nil, err
+	}
+	return v, nil
+}
+
 // convertPartition converts and appends the partition block into the table attributes if exists.
 func convertPartition(spec schemahcl.Resource, table *schema.Table) error {
 	r, ok := spec.Resource("partition")
@@ -424,16 +449,26 @@ func convertColumnType(spec *sqlspec.Column) (schema.Type, error) {
 // convertEnums converts possibly referenced column types (like enums) to
 // an actual schema.Type and sets it on the correct schema.Column.
 func convertEnums(tables []*sqlspec.Table, enums []*Enum, r *schema.Realm) error {
-	var (
-		used   = make(map[*Enum]struct{})
-		byName = make(map[string]*Enum)
-	)
+	byName := make(map[string]*schema.EnumType)
 	for _, e := range enums {
-		byName[e.Name] = e
+		if byName[e.Name] != nil {
+			return fmt.Errorf("duplicate enum %q", e.Name)
+		}
+		ns, err := specutil.SchemaName(e.Schema)
+		if err != nil {
+			return fmt.Errorf("extract schema name from enum reference: %w", err)
+		}
+		es, ok := r.Schema(ns)
+		if !ok {
+			return fmt.Errorf("schema %q defined on enum %q was not found in realm", ns, e.Name)
+		}
+		e1 := &schema.EnumType{T: e.Name, Schema: es, Values: e.Values}
+		es.Objects = append(es.Objects, e1)
+		byName[e.Name] = e1
 	}
 	for _, t := range tables {
 		for _, c := range t.Columns {
-			var enum *Enum
+			var enum *schema.EnumType
 			switch {
 			case c.Type.IsRef:
 				n, err := enumName(c.Type)
@@ -442,7 +477,7 @@ func convertEnums(tables []*sqlspec.Table, enums []*Enum, r *schema.Realm) error
 				}
 				e, ok := byName[n]
 				if !ok {
-					return fmt.Errorf("enum %q was not found", n)
+					return fmt.Errorf("enum %q was not found in realm", n)
 				}
 				enum = e
 			default:
@@ -452,18 +487,9 @@ func convertEnums(tables []*sqlspec.Table, enums []*Enum, r *schema.Realm) error
 				}
 				enum = byName[n]
 			}
-			used[enum] = struct{}{}
-			schemaE, err := specutil.SchemaName(enum.Schema)
-			if err != nil {
-				return fmt.Errorf("extract schema name from enum refrence: %w", err)
-			}
-			es, ok := r.Schema(schemaE)
-			if !ok {
-				return fmt.Errorf("schema %q not found in realm for table %q", schemaE, t.Name)
-			}
 			schemaT, err := specutil.SchemaName(t.Schema)
 			if err != nil {
-				return fmt.Errorf("extract schema name from table refrence: %w", err)
+				return fmt.Errorf("extract schema name from table reference: %w", err)
 			}
 			ts, ok := r.Schema(schemaT)
 			if !ok {
@@ -477,20 +503,14 @@ func convertEnums(tables []*sqlspec.Table, enums []*Enum, r *schema.Realm) error
 			if !ok {
 				return fmt.Errorf("column %q not found in table %q", c.Name, t.Name)
 			}
-			e := &schema.EnumType{T: enum.Name, Schema: es, Values: enum.Values}
 			switch t := cc.Type.Type.(type) {
 			case *ArrayType:
-				t.Type = e
+				t.Type = enum
 			default:
-				cc.Type.Type = e
+				cc.Type.Type = enum
 			}
 		}
 	}
-	for _, e := range enums {
-		if _, ok := used[e]; !ok {
-			return fmt.Errorf("enum %q declared but not used", e.Name)
-		}
-	}
 	return nil
 }
 
@@ -511,26 +531,24 @@ func enumRef(n string) *schemahcl.Ref {
 }
 
 // schemaSpec converts from a concrete Postgres schema to Atlas specification.
-func schemaSpec(schem *schema.Schema) (*doc, error) {
-	s, tbls, err := specutil.FromSchema(schem, tableSpec)
+func schemaSpec(s *schema.Schema) (*doc, error) {
+	spec, err := specutil.FromSchema(s, tableSpec, viewSpec)
 	if err != nil {
 		return nil, err
 	}
 	d := &doc{
-		Tables:  tbls,
-		Schemas: []*sqlspec.Schema{s},
+		Tables:  spec.Tables,
+		Views:   spec.Views,
+		Schemas: []*sqlspec.Schema{spec.Schema},
+		Enums:   make([]*Enum, 0, len(s.Objects)),
 	}
-	enums := make(map[string]bool)
-	for _, t := range schem.Tables {
-		for _, c := range t.Columns {
-			if e, ok := hasEnumType(c); ok && !enums[e.T] {
-				d.Enums = append(d.Enums, &Enum{
-					Name:   e.T,
-					Schema: specutil.SchemaRef(s.Name),
-					Values: e.Values,
-				})
-				enums[e.T] = true
-			}
+	for _, o := range s.Objects {
+		if e, ok := o.(*schema.EnumType); ok {
+			d.Enums = append(d.Enums, &Enum{
+				Name:   e.T,
+				Values: e.Values,
+				Schema: specutil.SchemaRef(spec.Schema.Name),
+			})
 		}
 	}
 	return d, nil
@@ -540,7 +558,7 @@ func schemaSpec(schem *schema.Schema) (*doc, error) {
 func tableSpec(table *schema.Table) (*sqlspec.Table, error) {
 	spec, err := specutil.FromTable(
 		table,
-		columnSpec,
+		tableColumnSpec,
 		pkSpec,
 		indexSpec,
 		specutil.FromForeignKey,
@@ -555,6 +573,17 @@ func tableSpec(table *schema.Table) (*sqlspec.Table, error) {
 	return spec, nil
 }
 
+// viewSpec converts from a concrete PostgreSQL schema.View to a sqlspec.View.
+func viewSpec(view *schema.View) (*sqlspec.View, error) {
+	spec, err := specutil.FromView(view, func(c *schema.Column, _ *schema.View) (*sqlspec.Column, error) {
+		return specutil.FromColumn(c, columnTypeSpec)
+	})
+	if err != nil {
+		return nil, err
+	}
+	return spec, nil
+}
+
 func pkSpec(idx *schema.Index) (*sqlspec.PrimaryKey, error) {
 	spec, err := specutil.FromPrimaryKey(idx)
 	if err != nil {
@@ -611,8 +640,8 @@ func partAttr(idx *schema.Index, part *schema.IndexPart, spec *sqlspec.IndexPart
 	return nil
 }
 
-// columnSpec converts from a concrete Postgres schema.Column into a sqlspec.Column.
-func columnSpec(c *schema.Column, _ *schema.Table) (*sqlspec.Column, error) {
+// tableColumnSpec converts from a concrete Postgres schema.Column into a sqlspec.Column.
+func tableColumnSpec(c *schema.Column, _ *schema.Table) (*sqlspec.Column, error) {
 	s, err := specutil.FromColumn(c, columnTypeSpec)
 	if err != nil {
 		return nil, err
@@ -695,17 +724,17 @@ var TypeRegistry = schemahcl.NewRegistry(
 		schemahcl.NewTypeSpec(TypePoint),
 		schemahcl.NewTypeSpec(TypePolygon),
 		schemahcl.NewTypeSpec(TypeDate),
-		schemahcl.NewTypeSpec(TypeTime, schemahcl.WithAttributes(precisionTypeAttr()), formatTime()),
-		schemahcl.NewTypeSpec(TypeTimeTZ, schemahcl.WithAttributes(precisionTypeAttr()), formatTime()),
-		schemahcl.NewTypeSpec(TypeTimestampTZ, schemahcl.WithAttributes(precisionTypeAttr()), formatTime()),
-		schemahcl.NewTypeSpec(TypeTimestamp, schemahcl.WithAttributes(precisionTypeAttr()), formatTime()),
+		schemahcl.NewTypeSpec(TypeTime, schemahcl.WithAttributes(schemahcl.PrecisionTypeAttr()), formatTime()),
+		schemahcl.NewTypeSpec(TypeTimeTZ, schemahcl.WithAttributes(schemahcl.PrecisionTypeAttr()), formatTime()),
+		schemahcl.NewTypeSpec(TypeTimestampTZ, schemahcl.WithAttributes(schemahcl.PrecisionTypeAttr()), formatTime()),
+		schemahcl.NewTypeSpec(TypeTimestamp, schemahcl.WithAttributes(schemahcl.PrecisionTypeAttr()), formatTime()),
 		schemahcl.AliasTypeSpec("double_precision", TypeDouble),
 		schemahcl.NewTypeSpec(TypeReal),
-		schemahcl.NewTypeSpec(TypeFloat, schemahcl.WithAttributes(precisionTypeAttr())),
+		schemahcl.NewTypeSpec(TypeFloat, schemahcl.WithAttributes(schemahcl.PrecisionTypeAttr())),
 		schemahcl.NewTypeSpec(TypeFloat8),
 		schemahcl.NewTypeSpec(TypeFloat4),
-		schemahcl.NewTypeSpec(TypeNumeric, schemahcl.WithAttributes(precisionTypeAttr(), &schemahcl.TypeAttr{Name: "scale", Kind: reflect.Int, Required: false})),
-		schemahcl.NewTypeSpec(TypeDecimal, schemahcl.WithAttributes(precisionTypeAttr(), &schemahcl.TypeAttr{Name: "scale", Kind: reflect.Int, Required: false})),
+		schemahcl.NewTypeSpec(TypeNumeric, schemahcl.WithAttributes(schemahcl.PrecisionTypeAttr(), schemahcl.ScaleTypeAttr())),
+		schemahcl.NewTypeSpec(TypeDecimal, schemahcl.WithAttributes(schemahcl.PrecisionTypeAttr(), schemahcl.ScaleTypeAttr())),
 		schemahcl.NewTypeSpec(TypeSmallSerial),
 		schemahcl.NewTypeSpec(TypeSerial),
 		schemahcl.NewTypeSpec(TypeBigSerial),
@@ -778,7 +807,7 @@ var TypeRegistry = schemahcl.NewRegistry(
 			}),
 		}
 		for _, f := range []string{"interval", "second", "day to second", "hour to second", "minute to second"} {
-			specs = append(specs, schemahcl.NewTypeSpec(specutil.Var(f), append(opts, schemahcl.WithAttributes(precisionTypeAttr()))...))
+			specs = append(specs, schemahcl.NewTypeSpec(specutil.Var(f), append(opts, schemahcl.WithAttributes(schemahcl.PrecisionTypeAttr()))...))
 		}
 		for _, f := range []string{"year", "month", "day", "hour", "minute", "year to month", "day to hour", "day to minute", "hour to minute"} {
 			specs = append(specs, schemahcl.NewTypeSpec(specutil.Var(f), opts...))
@@ -787,14 +816,6 @@ var TypeRegistry = schemahcl.NewRegistry(
 	}()...),
 )
 
-func precisionTypeAttr() *schemahcl.TypeAttr {
-	return &schemahcl.TypeAttr{
-		Name:     "precision",
-		Kind:     reflect.Int,
-		Required: false,
-	}
-}
-
 func attr(typ *schemahcl.Type, key string) (*schemahcl.Attr, bool) {
 	for _, a := range typ.Attrs {
 		if a.K == key {
