@@ -102,9 +102,8 @@ enum "account_type" {
 	var s schema.Schema
 	err := EvalHCLBytes([]byte(f), &s, nil)
 	require.NoError(t, err)
-	exp := &schema.Schema{
-		Name: "schema",
-	}
+	exp := schema.New("schema")
+	exp.AddObjects(&schema.EnumType{T: "account_type", Values: []string{"private", "business"}, Schema: exp})
 	exp.Tables = []*schema.Table{
 		{
 			Name:   "table",
@@ -263,6 +262,221 @@ enum "account_type" {
 	require.EqualValues(t, exp, &s)
 }
 
+func TestMarshalSpec_Schema(t *testing.T) {
+	s := schema.New("public").SetComment("schema comment")
+	buf, err := MarshalHCL(s)
+	require.NoError(t, err)
+	require.Equal(t, `schema "public" {
+  comment = "schema comment"
+}
+`, string(buf))
+
+	r := schema.NewRealm(
+		schema.New("s1").SetComment("c1"),
+		schema.New("s2").SetComment("c2"),
+		schema.New("s3"),
+	)
+	buf, err = MarshalHCL(r)
+	require.NoError(t, err)
+	require.Equal(t, `schema "s1" {
+  comment = "c1"
+}
+schema "s2" {
+  comment = "c2"
+}
+schema "s3" {
+}
+`, string(buf))
+}
+
+func TestUnmarshalSpec_Schema(t *testing.T) {
+	var (
+		s schema.Schema
+		f = `
+schema "public" {
+  comment = "schema comment"
+}
+`
+	)
+	require.NoError(t, EvalHCLBytes([]byte(f), &s, nil))
+	require.Equal(t, "public", s.Name)
+	require.Len(t, s.Attrs, 1)
+	require.Equal(t, "schema comment", s.Attrs[0].(*schema.Comment).Text)
+}
+
+func TestMarshalViews(t *testing.T) {
+	s := schema.New("public").
+		AddTables(
+			schema.NewTable("t1").
+				AddColumns(
+					schema.NewIntColumn("id", "int"),
+				),
+		).
+		AddViews(
+			schema.NewView("v1", "SELECT 1").
+				SetCheckOption(schema.ViewCheckOptionLocal),
+			schema.NewView("v2", "SELECT * FROM t2\n\tWHERE id IS NOT NULL"),
+			schema.NewView("v3", "SELECT * FROM t3\n\tWHERE id IS NOT NULL\n\tORDER BY id").
+				AddColumns(
+					schema.NewIntColumn("id", "id"),
+				).
+				SetComment("view comment"),
+		)
+	s.AddViews(
+		schema.NewView("v4", "SELECT * FROM v2 JOIN t1 USING (id)").
+			AddDeps(
+				s.Views[1],
+				s.Tables[0],
+			),
+	)
+	buf, err := MarshalHCL(s)
+	require.NoError(t, err)
+	f := `table "t1" {
+  schema = schema.public
+  column "id" {
+    null = false
+    type = int
+  }
+}
+view "v1" {
+  schema       = schema.public
+  as           = "SELECT 1"
+  check_option = LOCAL
+}
+view "v2" {
+  schema = schema.public
+  as     = <<-SQL
+  SELECT * FROM t2
+  	WHERE id IS NOT NULL
+  SQL
+}
+view "v3" {
+  schema = schema.public
+  column "id" {
+    null = false
+    type = sql("id")
+  }
+  as      = <<-SQL
+  SELECT * FROM t3
+  	WHERE id IS NOT NULL
+  	ORDER BY id
+  SQL
+  comment = "view comment"
+}
+view "v4" {
+  schema     = schema.public
+  as         = "SELECT * FROM v2 JOIN t1 USING (id)"
+  depends_on = [view.v2, table.t1]
+}
+schema "public" {
+}
+`
+	require.Equal(t, f, string(buf))
+}
+
+func TestUnmarshalViews(t *testing.T) {
+	f := `table "t1" {
+  schema = schema.public
+  column "id" {
+    null = false
+    type = int
+  }
+}
+view "v1" {
+  schema = schema.public
+  as     = "SELECT * FROM t2 WHERE id IS NOT NULL"
+}
+view "v2" {
+ schema = schema.public
+ column "id" {
+   null = false
+   type = int
+ }
+ as      = "SELECT * FROM t3 WHERE id IS NOT NULL ORDER BY id"
+ comment = "view comment"
+}
+view "v3" {
+ schema       = schema.public
+ as           = "SELECT * FROM v2 JOIN t1 USING (id)"
+ check_option = LOCAL
+ depends_on   = [view.v1, table.t1]
+}
+
+table "public" "t2" {
+  schema = schema.public
+  column "id" {
+    type = int
+  }
+}
+
+table "other" "t2" {
+  schema = schema.other
+  column "id" {
+    type = int
+  }
+}
+
+view "public" "v4" {
+  schema = schema.public
+  as     = "SELECT * FROM public.t2"
+  depends_on = [table.public.t2]
+}
+
+view "other" "v4" {
+  schema = schema.other
+  as     = "SELECT * FROM other.t2"
+  depends_on = [table.other.t2]
+}
+
+schema "public" {}
+schema "other" {}
+`
+	var (
+		r      schema.Realm
+		got    schema.Realm
+		public = schema.New("public").
+			AddTables(
+				schema.NewTable("t1").
+					AddColumns(
+						schema.NewIntColumn("id", "int"),
+					),
+				schema.NewTable("t2").
+					AddColumns(
+						schema.NewIntColumn("id", "int"),
+					),
+			).
+			AddViews(
+				schema.NewView("v1", "SELECT * FROM t2 WHERE id IS NOT NULL"),
+				schema.NewView("v2", "SELECT * FROM t3 WHERE id IS NOT NULL ORDER BY id").
+					AddColumns(
+						schema.NewIntColumn("id", "int"),
+					).
+					SetComment("view comment"),
+			)
+		other = schema.New("other").
+			AddTables(
+				schema.NewTable("t2").
+					AddColumns(
+						schema.NewIntColumn("id", "int"),
+					),
+			)
+	)
+	public.AddViews(
+		schema.NewView("v3", "SELECT * FROM v2 JOIN t1 USING (id)").
+			SetCheckOption(schema.ViewCheckOptionLocal).
+			AddDeps(public.Views[0], public.Tables[0]),
+		schema.NewView("v4", "SELECT * FROM public.t2").
+			AddDeps(public.Tables[1]),
+	)
+	other.AddViews(
+		schema.NewView("v4", "SELECT * FROM other.t2").
+			AddDeps(other.Tables[0]),
+	)
+	r.AddSchemas(public, other)
+	require.NoError(t, EvalHCLBytes([]byte(f), &got, nil))
+	require.EqualValues(t, r, got)
+}
+
 func TestUnmarshalSpec_IndexType(t *testing.T) {
 	f := `
 schema "s" {}
@@ -1075,7 +1289,18 @@ table "users" {
 }
 
 func TestMarshalSpec_Enum(t *testing.T) {
+	stateE := &schema.EnumType{
+		T:      "state",
+		Values: []string{"on", "off"},
+	}
+	typeE := &schema.EnumType{
+		T:      "account_type",
+		Values: []string{"private", "business"},
+	}
 	s := schema.New("test").
+		AddObjects(
+			typeE, stateE,
+		).
 		AddTables(
 			schema.NewTable("account").
 				AddColumns(
@@ -1085,19 +1310,14 @@ func TestMarshalSpec_Enum(t *testing.T) {
 					),
 					schema.NewColumn("account_states").
 						SetType(&ArrayType{
-							T: "states[]",
-							Type: &schema.EnumType{
-								T:      "state",
-								Values: []string{"on", "off"},
-							},
+							T:    "states[]",
+							Type: stateE,
 						}),
 				),
 			schema.NewTable("table2").
 				AddColumns(
-					schema.NewEnumColumn("account_type",
-						schema.EnumName("account_type"),
-						schema.EnumValues("private", "business"),
-					),
+					schema.NewColumn("account_type").
+						SetType(typeE),
 				),
 		)
 	buf, err := MarshalSpec(s, hclState)
@@ -1753,9 +1973,17 @@ func TestMarshalRealm(t *testing.T) {
 	// Reference is qualified with s1.
 	t5.AddForeignKeys(schema.NewForeignKey("oid2id2").AddColumns(t5.Columns[0]).SetRefTable(t2).AddRefColumns(t2.Columns[0]))
 
+	// Two views with the same name resided in different schemas.
+	v2 := schema.NewView("v2", "SELECT oid FROM s1.t2").
+		AddColumns(schema.NewIntColumn("oid", "int")).
+		AddDeps(t2)
+	v4 := schema.NewView("v2", "SELECT oid FROM s2.t2").
+		AddColumns(schema.NewIntColumn("oid", "int")).
+		AddDeps(t4)
+
 	r := schema.NewRealm(
-		schema.New("s1").AddTables(t1, t2),
-		schema.New("s2").AddTables(t3, t4, t5),
+		schema.New("s1").AddTables(t1, t2).AddViews(v2),
+		schema.New("s2").AddTables(t3, t4, t5).AddViews(v4),
 	)
 	got, err := MarshalHCL.MarshalSpec(r)
 	require.NoError(t, err)
@@ -1814,6 +2042,24 @@ table "t5" {
     ref_columns = [table.s1.t2.column.oid]
   }
 }
+view "s1" "v2" {
+  schema = schema.s1
+  column "oid" {
+    null = false
+    type = int
+  }
+  as         = "SELECT oid FROM s1.t2"
+  depends_on = [table.s1.t2]
+}
+view "s2" "v2" {
+  schema = schema.s2
+  column "oid" {
+    null = false
+    type = int
+  }
+  as         = "SELECT oid FROM s2.t2"
+  depends_on = [table.s2.t2]
+}
 schema "s1" {
 }
 schema "s2" {
