@@ -18,7 +18,7 @@ import (
 )
 
 // A diff provides an SQLite implementation for schema.Inspector.
-type inspect struct{ conn }
+type inspect struct{ *conn }
 
 var _ schema.Inspector = (*inspect)(nil)
 
@@ -35,22 +35,26 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 		opts = &schema.InspectRealmOption{}
 	}
 	r := schema.NewRealm(schemas...)
-	if !sqlx.ModeInspectRealm(opts).Is(schema.InspectTables) {
-		return sqlx.ExcludeRealm(r, opts.Exclude)
-	}
-	for _, s := range schemas {
-		tables, err := i.tables(ctx, nil)
-		if err != nil {
-			return nil, err
-		}
-		s.AddTables(tables...)
-		for _, t := range tables {
-			if err := i.inspectTable(ctx, t); err != nil {
+	if sqlx.ModeInspectRealm(opts).Is(schema.InspectTables) {
+		for _, s := range schemas {
+			tables, err := i.tables(ctx, nil)
+			if err != nil {
 				return nil, err
 			}
+			s.AddTables(tables...)
+			for _, t := range tables {
+				if err := i.inspectTable(ctx, t); err != nil {
+					return nil, err
+				}
+			}
+		}
+		sqlx.LinkSchemaTables(r.Schemas)
+	}
+	if sqlx.ModeInspectRealm(opts).Is(schema.InspectViews) {
+		if err := i.inspectViews(ctx, r, nil); err != nil {
+			return nil, err
 		}
 	}
-	sqlx.LinkSchemaTables(r.Schemas)
 	return sqlx.ExcludeRealm(r, opts.Exclude)
 }
 
@@ -75,20 +79,24 @@ func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.I
 		opts = &schema.InspectOptions{}
 	}
 	r := schema.NewRealm(schemas...)
-	if !sqlx.ModeInspectSchema(opts).Is(schema.InspectTables) {
-		return sqlx.ExcludeSchema(r.Schemas[0], opts.Exclude)
+	if sqlx.ModeInspectSchema(opts).Is(schema.InspectTables) {
+		tables, err := i.tables(ctx, opts)
+		if err != nil {
+			return nil, err
+		}
+		r.Schemas[0].AddTables(tables...)
+		for _, t := range tables {
+			if err := i.inspectTable(ctx, t); err != nil {
+				return nil, err
+			}
+		}
+		sqlx.LinkSchemaTables(schemas)
 	}
-	tables, err := i.tables(ctx, opts)
-	if err != nil {
-		return nil, err
-	}
-	r.Schemas[0].AddTables(tables...)
-	for _, t := range tables {
-		if err := i.inspectTable(ctx, t); err != nil {
+	if sqlx.ModeInspectSchema(opts).Is(schema.InspectViews) {
+		if err := i.inspectViews(ctx, r, opts); err != nil {
 			return nil, err
 		}
 	}
-	sqlx.LinkSchemaTables(schemas)
 	return sqlx.ExcludeSchema(r.Schemas[0], opts.Exclude)
 }
 
@@ -363,7 +371,7 @@ func (i *inspect) tables(ctx context.Context, opts *schema.InspectOptions) ([]*s
 		query = tablesQuery
 	)
 	if opts != nil && len(opts.Tables) > 0 {
-		query += " AND name IN (" + strings.Repeat("?, ", len(opts.Tables)-1) + "?)"
+		query += " AND sqlite_master.name IN (" + strings.Repeat("?, ", len(opts.Tables)-1) + "?)"
 		for _, s := range opts.Tables {
 			args = append(args, s)
 		}
@@ -375,8 +383,11 @@ func (i *inspect) tables(ctx context.Context, opts *schema.InspectOptions) ([]*s
 	defer rows.Close()
 	var tables []*schema.Table
 	for rows.Next() {
-		var name, stmt string
-		if err := rows.Scan(&name, &stmt); err != nil {
+		var (
+			name, stmt string
+			wr, strict sql.NullBool
+		)
+		if err := rows.Scan(&name, &stmt, &wr, &strict); err != nil {
 			return nil, fmt.Errorf("sqlite: scanning table: %w", err)
 		}
 		stmt = strings.TrimSpace(stmt)
@@ -386,9 +397,12 @@ func (i *inspect) tables(ctx context.Context, opts *schema.InspectOptions) ([]*s
 				&CreateStmt{S: strings.TrimSpace(stmt)},
 			},
 		}
-		if strings.HasSuffix(stmt, "WITHOUT ROWID") || strings.HasSuffix(stmt, "without rowid") {
+		if wr.Bool {
 			t.Attrs = append(t.Attrs, &WithoutRowID{})
 		}
+		if strict.Bool {
+			t.Attrs = append(t.Attrs, &Strict{})
+		}
 		tables = append(tables, t)
 	}
 	return tables, nil
@@ -462,6 +476,12 @@ type (
 		schema.Attr
 	}
 
+	// Strict describes the `STRICT` configuration.
+	// See: https://sqlite.org/stricttables.html
+	Strict struct {
+		schema.Attr
+	}
+
 	// IndexPredicate describes a partial index predicate.
 	// See: https://www.sqlite.org/partialindex.html
 	IndexPredicate struct {
@@ -715,7 +735,17 @@ const (
 	databasesQuery     = "SELECT `name`, `file` FROM pragma_database_list() WHERE `name` <> 'temp'"
 	databasesQueryArgs = "SELECT `name`, `file` FROM pragma_database_list() WHERE `name` IN (%s)"
 	// Query to list database tables.
-	tablesQuery = "SELECT `name`, `sql` FROM sqlite_master WHERE `type` = 'table' AND `name` NOT LIKE 'sqlite_%'"
+	tablesQuery = `
+SELECT
+	sqlite_master.name, sqlite_master.sql, wr, strict
+FROM
+	sqlite_master
+	JOIN pragma_table_list(sqlite_master.name)
+WHERE
+	sqlite_master.type = 'table'
+	AND sqlite_master.name NOT LIKE 'sqlite_%'
+	AND sqlite_master.name NOT LIKE 'libsql_%'
+`
 	// Query to list table information.
 	columnsQuery = "SELECT `name`, `type`, (not `notnull`) AS `nullable`, `dflt_value`, (`pk` <> 0) AS `pk`, `hidden` FROM pragma_table_xinfo('%s') ORDER BY `cid`"
 	// Query to list table indexes.
